function x=marqlev(critfun,x0,param)
% Marquardt-Levenberg like minimization method
% critfun - criterion function, e.g. 'crit', will be called
% with two arguments - a vector x and additional parameters param,
% or just with x, if param is empty.
% critfun should return three output arguments: 
% the criterion, its gradient and its Hessian
% x0 is the starting guess
%
% Usage: x=marqlev(critfun,x0,[param])
%
% $Id: marqlev.m,v 1.12 2000/09/01 09:50:55 jkybic Exp $
% Jan Kybic, 1999

x=x0(:) ;
N=length(x) ;

if nargin<3, param=[] ; end ;

prec=1e-3 ;   % Relative precision
delta=1e-20 ; % Absolute precision
lambda=10 ;   % Initial lambda
condn=1e-10 ; % Minimum conditioning
%lambdaf=10 ;  % Lambda factor
lambdaf=2 ;  % Lambda factor
maxlambda=1e50 ; 
displaying=1 ;
dofine=0 ;
j=0 ; % a counter of succesful steps
i=1 ; % counter of evaluations

[Eold,gr,H]=fevalpar(critfun,x,param) ; % initial evaluation
if displaying,
  disp([ 'Iteration ' num2str(i) '  criterion=' num2str(Eold)]) ;
end ;
if dofine,
  evalfinecrit(x,i) ;
end ;

i=i+1 ;

E=Eold ;
Eold=2*Eold+1 ; % just a hack to avoid stopping in the first iteration

while ((Eold-E>Eold*prec+delta | j<1) & lambda<maxlambda),
  E1=E ;
  M=H+eye(N)*lambda ; rc=rcond(M) ;
  % M=diag(diag(M)) ; % consider only the diagonal
  if rc>condn, % if the matrix is well conditioned
    cc=cputime ;
    dx=-M\gr ;
    if displaying,
      disp([ 'linear system solving took ' num2str(cputime-cc) ' s' ]) ;
    end ;
    xpdx=x+dx ;
    [E,gr1,H1]=fevalpar(critfun,xpdx,param) ;
    if displaying,
      disp([ 'Iteration ' num2str(i) '  criterion=' num2str(E) ' lambda=' num2str(lambda)]) ;
    end ;
    if dofine,
      evalfinecrit(xpdx,i) ;
    end ;
    i=i+1; 
  end ; % if the matrix is ill-conditioned, do not do anything
  if E>E1 | rc<=condn, % if the step failed
    lambda=lambdaf*lambda ; E=E1 ;
    if displaying,
      disp([ 'Step failed, increasing lambda to ' num2str(lambda) ]) ;
    end ;
    j=0 ;
  else % successful steps
    lambda=lambda/lambdaf ; x=xpdx ;
    Eold=E1 ; H=H1 ; gr=gr1 ;
    j=j+1 ;
  end ;
end ;

if displaying,
  disp([ 'Iteration ' num2str(i) '  criterion=' num2str(E) ' lambda=' num2str(lambda)]) ;
end ;
  
%----------------------------------------------------------------------

function [J,gr,H]=fevalpar(critfun,x,param)
if isempty(param),
  [J,gr,H]=feval(critfun,x) ;
else
  [J,gr,H]=feval(critfun,x,param) ;
end ;
gr=gr(:) ;
