function [c,cm]=bpgetcoeffdi(thetav,uv,mts,T,gamma)
% Calculates the reconstruction coefficients using turned and shifted 
% functions optimal for the finite domain integration
% interpolation for the tomographical reconstruction problem
%
% Usage: c=bpgetcoeffdi(thetav,uv,mts,T[,gamma])
%
% T is the integration length, thetav angles, uv offsets, mts measurements
% measurements mts should be of the size zeros(length(thetav),length(uv)) 
%
% Jan Kybic, March 2000

nt=length(thetav) ;
nu=length(uv) ;

if nargin<5, gamma=0 ; end ;

% setup the system matrix
N=nu*nt+3 ; % number of unknowns
if 0, % Matlab code  versus C implementation
  A=zeros(N,N) ;
  for i1=1:nu, % sampling point
    for i2=1:nt,
      i=i1+nu*(i2-1) ;
      limT=sqrt(T^2-uv(i1)^2) ;
      for j1=1:nu, % coefficient index
	for j2=1:nt,
	  j=j1+nu*(j2-1) ;
	  %	A(i,j)=intxthr(thetav(j2),thetav(i2),uv(j1),uv(i1),T) ;
	  %	A(i,j)=intxthr(thetav(j2),thetav(i2),uv(j1),uv(i1),limT) ;
	  A(i,j)=intfdi(thetav(i2),thetav(j2),uv(i1),uv(j1),T) ;
	end ;
      end ;
      %    [val1,valx,valy]=intlpol(thetav(i2),uv(i1),T) ;
      [val1,valx,valy]=intlpol(thetav(i2),uv(i1),limT) ;
      A(i,nu*nt+1)=val1 ;
      A(i,nu*nt+2)=valx ;
      A(i,nu*nt+3)=valy ;
    end ;
  end ;

  A(nu*nt+1:nu*nt+3,:)=A(:,nu*nt+1:nu*nt+3)' ;
  
else
  A=mexbpmatfdi(thetav,uv,T) ;
end ;

r=[ reshape(mts',nu*nt,1) ; 0 ; 0 ; 0 ] ;
NN=nu*nt ;
max(max(abs(A)))
%keyboard
A(1:NN,1:NN)=A(1:NN,1:NN)+eye(NN)*gamma ; % regularization
disp('Matrix known, now solving') ;
c=A\r ;

cm=A*c ;
res=max(abs(cm-r))  % the residual


function val=intfdi(th1,th2,u1,u2,T) ;
% Calculates the integral of the finite domain optimal function
% corresponding to angle th2 and offset u2 along a line with angle
% th1 and offset u1. All inside a circle of diameter T
thdif=th1-th2 ;
st=sqrt(T^2-u1^2) ;
val=intI(thdif,u1,u2,T,st)-intI(thdif,u1,u2,T,-st) ;


function val=intxthr(th1,th2,u1,u2,T) ;
% Integrate |-sin(th1)*x+cos(th1)*y-u1|^3
% along (cos(th2)*t-sin(th2)*u2,sin(th2)*t+cos(th2)*u2) for t=-T..T
thdif=th1-th2 ;
alpha=abs(sin(thdif)) ; beta=cos(thdif)*u2-u1 ;
if alpha==0,
  val=T*abs(beta)^3/6 ;
elseif -alpha*T+beta>0,
  val=(T*beta*(alpha^2*T^2+beta^2))/6 ;
elseif alpha*T+beta<0,  
  val=-(T*beta*(alpha^2*T^2+beta^2))/6 ;
else
  val=((alpha*T+beta)^4+(-alpha*T+beta)^4)/4/abs(alpha)/12 ;
end ;

function val=intI(thdif, q,t,T,p) ;

% disp(sprintf('intI: thdif=%g q=%g t=%g T=%g p=%g',thdif,q,t,T,p))
sint=sin(thdif) ; cost=cos(thdif) ;
alpha2=cost ; alpha3=-q*sint ; % I change here x and y
alpha0=sint ; alpha1=q*cost ;
if alpha0~=0,
  val=intv1(alpha2/alpha0,-alpha2*alpha1/alpha0+alpha3,...
      p*alpha0+alpha1,t,T)/alpha0 ;
else
  val=intv2(alpha1,p*alpha2+alpha3,t,T)/alpha2 ;
end ;

function val=intv1(a,b,p,t,T)
% Evaluate primitive function to v(x,ax+b) at point x=p
% disp(sprintf('intv1: a=%g b=%g p=%g t=%g T=%g',a,b,p,t,T))

st=sqrt(T^2-t^2) ;
val=intu1(a,a*t+b+st,p-t)/2-intu1(a,a*t+b-st,p-t)/2 ;

function val=intv2(c,p,t,T)
% Evaluate primitive function to v(c,x) at point x=p
% disp(sprintf('intv2: c=%g p=%g t=%g T=%g',c,p,t,T))

st=sqrt(T^2-t^2) ;
val=intu2(c-t,p+st)/2-intu2(c-t,p-st)/2 ;

function val=intu1(a,b,p)
% Evaluate the primitive function of u(x,ax+b) at x=p
%if a<0, a=-a ; p=-p ; end ;
% disp(sprintf('intu1: a=%g b=%g p=%g',a,b,p))

if p~=0,
  val=(-24*p^4*atan(a+b/p)+...
      (-24*b^4*atan((p+a*(b+a*p))/b)-...
      6*a*(3+a^2)*b^4*...
      log(b^2+2*a*b*p+(1+a^2)*p^2)+...
      (1+a^2)*p*(12*(2+a^2)*b^3+18*(a+a^3)*b^2*p+4*(2+5*a^2+3*a^4)*b*p^2+...
      3*a*(3+4*a^2+a^4)*p^3-6*(1+a^2)*(4*b^3+6*a*b^2*p+4*(1+a^2)*b*p^2+...
      a*(3+a^2)*p^3)*log(p^2+(b+a*p)^2)))/(1+a^2)^2)/576/pi ;
else
  val=((24*b^4*atan(b/(a*b)))-6*a*(3+a^2)*b^4*log(b^2))/(1+a^2)^2/576/pi ;
end ;

function val=intu2(c,p)
% Evaluate the primitive function of u(c,x) at x=p
% disp(sprintf('intu2: c=%g p=%g',c,p))

if c~=0,
  val=(10*c^2*p^2+p^4-32*c^3*p*atan(p/c)-...
      2*(-3*c^4+6*c^2*p^2+p^4)*log(c^2+p^2))/192/pi ;
else
  if p~=0,
    val=(p^4-2*(p^4)*log(p^2))/192/pi ;
  else
     val=(p^4)/192/pi ;
  end ;
end ;  



 function [val1,valx,valy]=intlpol(th2,u2,T) ;
% Integrate (1,x,y) along
% (cos(th2)*t-sin(th2)*u2,sin(th2)*t+cos(th2)*u2) for t=-T..T
val1=2*T ;
valx=-2*T*sin(th2)*u2 ;
valy=+2*T*cos(th2)*u2 ;
