/*
  mexbpevalcbspln   

  Evaluates a linear combination of turned and shifted |x|^3 
  functions and evaluates it over an image. Point (0,0) is in the middle of the 
  image, pixels correspond to step 1.

  Usage: img=mexbpevalcbspln(thetav,nuh,du,c,dim) ;

  img...output image
  thetav...projection angles in radians
  uv...displacements for measures for each angle in pixel units
    there are 2*nuh-1 of them, centered at zero, with step du
  c...coefficients - there should be Ntheta*Nuv+3 of them
  dim...[dimy dimx] - size of the resulting image

  $Id: mexbpevalcbspln.c,v 1.2 2000/05/12 12:57:59 jkybic Exp $

  Jan Kybic, April 2000
*/

#include <math.h>
#include "mex.h"

/* define a few macros to keep the other source files generic */
#define myErrMsg mexErrMsgTxt
#define NAME "mexbpevalcbspln"
#define myMalloc(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(double)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myMallocInt(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(int)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myFree(x) mxFree(x)


int bpevalcbsplnimg(double *thetav,int nthetav,int nuh, double du,
                 double *c,int nc,double *img,
		 int dimy,int dimx) ;


/* Input Arguments */

#define	THETAV_IN	   prhs[0]
#define	NUH_IN             prhs[1]
#define DU_IN              prhs[2]
#define	C_IN	           prhs[3]
#define DIM_IN             prhs[4]

/* Output Arguments */

#define	IMG_OUT	   plhs[0]

void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{ double *thetav, *uv,*c,*img,du ;
  int i,nthetav,nuv,nc,dimy,dimx,nuh ;

  if (nrhs != 5) { /* Check for proper number of arguments */
    mexErrMsgTxt(NAME " requires five input arguments.");
  } else if (nlhs != 1) {
    mexErrMsgTxt(NAME " requires one output argument.");
  }

  for (i=0;i<nrhs;i++) /* Check for the type */
    { if (!mxIsNumeric(prhs[i]) || mxIsComplex(prhs[i]) || 
      mxIsSparse(prhs[i])  || !mxIsDouble(prhs[i])) 
         mexErrMsgTxt(NAME " needs all input arguments to be real.");
     } 
  
  thetav=mxGetPr(THETAV_IN) ;
  nuh=(int)(*(mxGetPr(NUH_IN))) ;
  c=mxGetPr(C_IN) ;
  dimx=(int)(*(mxGetPr(DIM_IN)+1)) ;
  dimy=(int)(*(mxGetPr(DIM_IN)+0)) ;
  nthetav=mxGetM(THETAV_IN)*mxGetN(THETAV_IN) ; 
  nc=mxGetM(C_IN)*mxGetN(C_IN) ; 
  du=*(mxGetPr(DU_IN)) ;

  if ((IMG_OUT=mxCreateDoubleMatrix(dimy,dimx,mxREAL))==NULL) 
    mexErrMsgTxt(NAME " Could not create array.") ;
  img=mxGetPr(IMG_OUT) ;

  /* mexPrintf("Calling bpevalthrimg with nthetav=%d nuv=%d nc=%d dimx=%d dimy=%d\n", nthetav,nuv,nc,dimx,dimy) ;
   */
  if (bpevalcbsplnimg(thetav,nthetav,nuh,du,c,nc,img,dimy,dimx))
    myErrMsg(NAME "Error in bpevalthrimg") ;
}

#include "bsplneval.c"

int bpevalcbsplnimg(double *thetav,int nthetav,int nuh,double du,
                 double *c,int nc,double *img,int dimy,int dimx)
{ int iu,it,ix,iy ;
  double tmp,val,x,y,cx,cy ;
  double theta,sint,cost ;
  int nuv ;

  nuv=2*nuh-1 ;
  if (nc!=nthetav*nuv) {
    mexPrintf(NAME " Incorrect length of the c vector\n") ;
    return -1 ;
  }
  cx=dimx/2 ; cy=dimy/2 ;
  for (ix=0;ix<dimx;ix++)
    for (iy=0;iy<dimy;iy++) {
      x=ix-cx ; y=iy-cy ;
      val=0.0 ; 
      for (it=0;it<nthetav;it++) {
	theta=thetav[it] ; sint=sin(theta) ; cost=cos(theta) ;
	for (iu=0;iu<nuv;iu++) {
	  tmp=(-sint*x+cost*y)/du-(1+iu-nuh) ;
	  val+=c[iu+it*nuv]*bspln3(tmp) ;
	}
      }
      img[iy+dimy*ix]=val ;
    }

  return 0 ;
}
