/*
  mexbpevalfdiimg   

  Evaluates a linear combination of turned and shifted |x|^3 
  functions and evaluates it over an image. Point (0,0) is in the middle of the 
  image, pixels correspond to step 1.

  Usage: img=mexbpevalfdiimg(thetav,uv,c,T,dim) ;

  img...output image
  thetav...projection angles in radians
  uv...displacements for measures for each angle in pixel units
    there are 2*nuh-1 of them, centered at zero, with step du
  c...coefficients - there should be Ntheta*Nuv+3 of them
  dim...[dimy dimx] - size of the resulting image

  $Id: mexbpevalfdiimg.c,v 1.1 2000/05/12 12:57:59 jkybic Exp $

  Jan Kybic, April 2000
*/

#include <math.h>
#include "mex.h"

/* define a few macros to keep the other source files generic */
#define myErrMsg mexErrMsgTxt
#define NAME "mexbpevalfdiimg"
#define myMalloc(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(double)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myMallocInt(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(int)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myFree(x) mxFree(x)


int bpevalfdiimg(double *thetav,int nthetav,double *uv,int nuv,
                 double *c,int nc,double *img,
		 int dimy,int dimx,double T) ;


/* Input Arguments */

#define	THETAV_IN	   prhs[0]
#define	UV_IN              prhs[1]
#define	C_IN	           prhs[2]
#define	T_IN	           prhs[3]
#define DIM_IN             prhs[4]

/* Output Arguments */

#define	IMG_OUT	   plhs[0]

void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{ double *thetav, *uv,*c,*img,T ;
  int i,nthetav,nuv,nc,dimy,dimx ;

  if (nrhs != 5) { /* Check for proper number of arguments */
    mexErrMsgTxt(NAME " requires six input arguments.");
  } else if (nlhs != 1) {
    mexErrMsgTxt(NAME " requires one output argument.");
  }

  for (i=0;i<nrhs;i++) /* Check for the type */
    { if (!mxIsNumeric(prhs[i]) || mxIsComplex(prhs[i]) || 
      mxIsSparse(prhs[i])  || !mxIsDouble(prhs[i])) 
         mexErrMsgTxt(NAME " needs all input arguments to be real.");
     } 
  
  thetav=mxGetPr(THETAV_IN) ;
  uv=mxGetPr(UV_IN) ;
  nuv=mxGetM(UV_IN)*mxGetN(UV_IN) ;
  c=mxGetPr(C_IN) ;
  dimx=(int)(*(mxGetPr(DIM_IN)+1)) ;
  dimy=(int)(*(mxGetPr(DIM_IN)+0)) ;
  nthetav=mxGetM(THETAV_IN)*mxGetN(THETAV_IN) ; 
  nc=mxGetM(C_IN)*mxGetN(C_IN) ; 
  T=*(mxGetPr(T_IN)) ;

  if ((IMG_OUT=mxCreateDoubleMatrix(dimy,dimx,mxREAL))==NULL) 
    mexErrMsgTxt(NAME " Could not create array.") ;
  img=mxGetPr(IMG_OUT) ;

  if (bpevalfdiimg(thetav,nthetav,uv,nuv,c,nc,img,dimy,dimx,T))
    myErrMsg(NAME "Error in bpevalthrimg") ;
}

static double valu(double xp,double yp)
{ double xp2,yp2 ;
  xp2=xp*xp ; yp2=yp*yp ; 
  if (xp!=0.0) 
    return -(0.5*yp*(3*xp2+yp2)*log(xp2+yp2)+2*xp2*xp*atan(yp/xp))/12.0/M_PI ;
  if (yp!=0)
    return -yp2*yp*log(yp2)/24/M_PI ;
  return 0.0 ;
}

static double valv(double xp,double yp,double t,double T) 
{ double st=sqrt(T*T-t*t) ;
  return 0.5*(valu(xp-t,yp+st)-valu(xp-t,yp-st)) ;
}


int bpevalfdiimg(double *thetav,int nthetav,double *uv,int nuv,
                 double *c,int nc,double *img,int dimy,int dimx,double T)
{ int iu,it,ix,iy ;
  double tmp,val,x,y,cx,cy,xp,yp ;
  double theta,sint,cost ;

  if (nc!=nthetav*nuv+3) {
    mexPrintf(NAME " Incorrect length of the c vector\n") ;
    return -1 ;
  }
  cx=dimx/2 ; cy=dimy/2 ;
  for (ix=0;ix<dimx;ix++)
    for (iy=0;iy<dimy;iy++) {
      x=ix-cx ; y=iy-cy ;
      if (x*x+y*y>T*T) val=0.0 ; else
	{
	  val=c[nc-3]+c[nc-2]*x+c[nc-1]*y ; 
	  for (it=0;it<nthetav;it++) {
	    theta=thetav[it] ; sint=sin(theta) ; cost=cos(theta) ;
	    xp=cost*x+y*sint ; yp=-sint*x+cost*y ;
	    for (iu=0;iu<nuv;iu++) {
	      val+=c[iu+it*nuv]*valv(yp,xp,uv[iu],T) ;
	    }
	  }
	}
      img[iy+dimy*ix]=val ;
    }

  return 0 ;
}
