/*
  mexbpevalthrimg   

  Evaluates a linear combination of turned and shifted |x|^3 
  functions and evaluates it over an image. Point (0,0) is in the middle of the 
  image, pixels correspond to step 1.

  Usage: img=mexbpevalthrimg(thetav,uv,c,dim,scl) ;

  img...output image
  thetav...projection angles in radians
  uv...displacements for measures for each angle in pixel units
  c...coefficients - there should be Ntheta*Nuv+3 of them
  dim...[dimy dimx] - size of the resulting image
  scl...scaling factor, to reconstruct a bigger image, call the routine
        using e.g. (...,dim*4,1/4)

  $Id: mexbpevalthrimg.c,v 1.3 2000/04/11 18:03:57 jkybic Exp $

  Jan Kybic, April 2000
*/

#include <math.h>
#include "mex.h"

/* define a few macros to keep the other source files generic */
#define myErrMsg mexErrMsgTxt
#define NAME "mexbpevalthrimg"
#define myMalloc(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(double)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myMallocInt(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(int)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myFree(x) mxFree(x)


int bpevalthrimg(double *thetav,int nthetav,double *uv,
                 int nuv,double *c,int nc,double *img,
		 int dimy,int dimx,double scl) ;


/* Input Arguments */

#define	THETAV_IN	   prhs[0]
#define	UV_IN              prhs[1]
#define	C_IN	           prhs[2]
#define DIM_IN             prhs[3]
#define SCL_IN             prhs[4]

/* Output Arguments */

#define	IMG_OUT	   plhs[0]

void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{ double *thetav, *uv,*c,*img,scl ;
  int i,nthetav,nuv,nc,dimy,dimx ;

  if (nrhs != 5) { /* Check for proper number of arguments */
    mexErrMsgTxt(NAME " requires five input arguments.");
  } else if (nlhs != 1) {
    mexErrMsgTxt(NAME " requires one output argument.");
  }

  for (i=0;i<nrhs;i++) /* Check for the type */
    { if (!mxIsNumeric(prhs[i]) || mxIsComplex(prhs[i]) || 
      mxIsSparse(prhs[i])  || !mxIsDouble(prhs[i])) 
         mexErrMsgTxt(NAME " needs all input arguments to be real.");
     } 
  
  thetav=mxGetPr(THETAV_IN) ;
  uv=mxGetPr(UV_IN) ;
  c=mxGetPr(C_IN) ;
  dimx=(int)(*(mxGetPr(DIM_IN)+1)) ;
  dimy=(int)(*(mxGetPr(DIM_IN)+0)) ;
  nthetav=mxGetM(THETAV_IN)*mxGetN(THETAV_IN) ; 
  nuv=mxGetM(UV_IN)*mxGetN(UV_IN) ; 
  nc=mxGetM(C_IN)*mxGetN(C_IN) ; 
  scl=*(mxGetPr(SCL_IN)) ;

  if ((IMG_OUT=mxCreateDoubleMatrix(dimy,dimx,mxREAL))==NULL) 
    mexErrMsgTxt(NAME " Could not create array.") ;
  img=mxGetPr(IMG_OUT) ;

  /* mexPrintf("Calling bpevalthrimg with nthetav=%d nuv=%d nc=%d dimx=%d dimy=%d\n", nthetav,nuv,nc,dimx,dimy) ;
   */
  if (bpevalthrimg(thetav,nthetav,uv,nuv,c,nc,img,dimy,dimx,scl))
    myErrMsg(NAME "Error in bpevalthrimg") ;
}

int bpevalthrimg(double *thetav,int nthetav,double *uv,
                 int nuv,double *c,int nc,double *img,int dimy,int dimx, 
		 double scl)
{ int iu,it,ix,iy ;
  double tmp,val,x,y,cx,cy ;
  double theta,sint,cost ;

  if (nc!=nthetav*nuv+3) {
    mexPrintf(NAME " Incorrect length of the c vector\n") ;
    return -1 ;
  }
  cx=dimx/2 ; cy=dimy/2 ;
  for (ix=0;ix<dimx;ix++)
    for (iy=0;iy<dimy;iy++) {
      x=(ix-cx)*scl ; y=(iy-cy)*scl ;
      val=c[nc-3]+c[nc-2]*x+c[nc-1]*y ; 
      for (it=0;it<nthetav;it++) {
	theta=thetav[it] ; sint=sin(theta) ; cost=cos(theta) ;
	for (iu=0;iu<nuv;iu++) {
	  tmp=fabs(-sint*x+cost*y-uv[iu]) ;
	  val+=c[iu+it*nuv]*fabs(tmp*tmp*tmp)/12 ;
	}
      }
      img[iy+dimy*ix]=val ;
    }

  return 0 ;
}
