/*
  mexbpmatcbspln

  Generates the system matrix for a reconstruction using
  a linear combination of turned and shifted |x|^3 
  functions. Point (0,0) is in the middle of the 
  image, pixels correspond to step 1.

  Usage: A=mexbpmatxthr(thetav,nuh,du,T) ;

  A...system matrix to find coefficients c=A\r
  thetav...projection angles in radians
  uv...displacements for measures for each angle in pixel units
  T...diametr of the circle inside which we integrate

  $Id: mexbpmatcbspln.c,v 1.1 2000/04/26 18:06:45 jkybic Exp $

  Jan Kybic, 2000
*/

#include <math.h>
#include "mex.h"

/* define a few macros to keep the other source files generic */
#define myErrMsg mexErrMsgTxt
#define NAME "mexbpmatcbspln"
#define myMalloc(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(double)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myMallocInt(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(int)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myFree(x) mxFree(x)


int bpmatcbspln(double *thetav,int nthetav,int nuh,double du,double T,
	      double *A,int nc) ;

/* Input Arguments */

#define	THETAV_IN	   prhs[0]
#define	NUH_IN             prhs[1]
#define DU_IN              prhs[2]
#define	T_IN	           prhs[3]

/* Output Arguments */

#define	A_OUT	   plhs[0]

void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{ double *thetav, du, T, *A ;
  int i,nthetav,nc,nuh,nuv ;

  if (nrhs != 4) { /* Check for proper number of arguments */
    mexErrMsgTxt(NAME " requires four input arguments.");
  } else if (nlhs != 1) {
    mexErrMsgTxt(NAME " requires one output argument.");
  }

  for (i=0;i<nrhs;i++) /* Check for the type */
    { if (!mxIsNumeric(prhs[i]) || mxIsComplex(prhs[i]) || 
      mxIsSparse(prhs[i])  || !mxIsDouble(prhs[i])) 
         mexErrMsgTxt(NAME " needs all input arguments to be real.");
     } 
  
  thetav=mxGetPr(THETAV_IN) ;
  du=*(mxGetPr(DU_IN)) ;
  nuh=(int)(*(mxGetPr(NUH_IN))) ;
  T=*(mxGetPr(T_IN)) ;
  nthetav=mxGetM(THETAV_IN)*mxGetN(THETAV_IN) ; 
  nuv=2*nuh-1 ; 
  nc=nthetav*nuv ;

  if ((A_OUT=mxCreateDoubleMatrix(nc,nc,mxREAL))==NULL) 
    mexErrMsgTxt(NAME " Could not create array.") ;
  A=mxGetPr(A_OUT) ;

  if (bpmatcbspln(thetav,nthetav,nuh,du,T,A,nc))
    myErrMsg(NAME "Error in bpmatxthr") ;
}

double intxthr(double sint,double cost,double juv,double iuv,double limT) 
{ double val,alpha,beta,tmp1,tmp2 ;
  double fzero=1e-10 ;

  alpha=fabs(sint) ; beta=cost*iuv-juv ;
  if (alpha<=fzero) {
    val=fabs(beta)*beta*beta ;
    val=limT*val/6 ;
  } else if (-alpha*limT+beta>0) 
    val=(limT*beta*(alpha*alpha*limT*limT+beta*beta))/6 ;
  else if (alpha*limT+beta<0)
    val=-(limT*beta*(alpha*alpha*limT*limT+beta*beta))/6 ;
  else {
    tmp1=alpha*limT+beta ; tmp2=-alpha*limT+beta ;
    tmp1=tmp1*tmp1 ; tmp1=tmp1*tmp1 ;
    tmp2=tmp2*tmp2 ; tmp2=tmp2*tmp2 ;
    val=(tmp1+tmp2)/alpha/48 ;
  }
  return val ;
}

int bpmatcbspln(double *thetav,int nthetav,int nuh,double du,double T,
	      double *A,int nc)

{ int iu,it,ju,jt,i,j ;
  double limT,val,thdif,sint,cost,alpha,beta ;
  double iuv,juv ;
  int nuv ;

  nuv=2*nuh-1 ;
  for (it=0;it<nthetav;it++)
    for (iu=0;iu<nuv;iu++) { /* sampling operator - rows */
      i=iu+nuv*it ;
      iuv=(1+iu-nuh)*du ;
      limT=T*T-iuv*iuv ;
      limT=limT>0 ? sqrt(limT) : 0 ;
      for (jt=0;jt<nthetav;jt++)
	for (ju=0;ju<nuv;ju++) { /* basis function - columns */
	  j=ju+nuv*jt ;
	  thdif=thetav[jt]-thetav[it] ;
	  sint=sin(thdif) ; cost=cos(thdif) ;
	  juv=(1+ju-nuh)*du ;
	  val=intxthr(sint,cost,juv-2*du,iuv,limT) ;
	  val=val-4*intxthr(sint,cost,juv-du,iuv,limT) ;
	  val=val+6*intxthr(sint,cost,juv,iuv,limT) ;
	  val=val-4*intxthr(sint,cost,juv+du,iuv,limT) ;
	  val=val+intxthr(sint,cost,juv+2*du,iuv,limT) ;
	  A[i+j*nc]=val/du/du/du ;
	} /* end of for ju, jt */
    } /* end of for iu,it */
  return 0 ;
}
