/*
  mexbpmatfdi

  Generates the system matrix for a reconstruction using
  a linear combination of turned and shifted |x|^3 
  functions. Point (0,0) is in the middle of the 
  image, pixels correspond to step 1.

  Usage: A=mexbpmatfdi(thetav,uv,T) ;

  A...system matrix to find coefficients c=A\r
  thetav...projection angles in radians
  uv...displacements for measures for each angle in pixel units
  T...diametr of the circle inside which we integrate

  $Id: mexbpmatfdi.c,v 1.2 2000/06/06 17:15:34 jkybic Exp $

  Jan Kybic, 2000
*/

#include <math.h>
#include "mex.h"

/* define a few macros to keep the other source files generic */
#define myErrMsg mexErrMsgTxt
#define NAME "mexbpmatfdi"
#define myMalloc(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(double)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myMallocInt(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(int)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myFree(x) mxFree(x)


int bpmatfdi(double *thetav,int nthetav,double *uv,int nuv,double T,
	      double *A,int nc) ;

/* Input Arguments */

#define	THETAV_IN	   prhs[0]
#define	UV_IN              prhs[1]
#define	T_IN	           prhs[2]

/* Output Arguments */

#define	A_OUT	   plhs[0]

void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{ double *thetav, *uv, T, *A ;
  int i,nthetav,nuv,nc ;

  if (nrhs != 3) { /* Check for proper number of arguments */
    mexErrMsgTxt(NAME " requires three input arguments.");
  } else if (nlhs != 1) {
    mexErrMsgTxt(NAME " requires one output argument.");
  }

  for (i=0;i<nrhs;i++) /* Check for the type */
    { if (!mxIsNumeric(prhs[i]) || mxIsComplex(prhs[i]) || 
      mxIsSparse(prhs[i])  || !mxIsDouble(prhs[i])) 
         mexErrMsgTxt(NAME " needs all input arguments to be real.");
     } 
  
  thetav=mxGetPr(THETAV_IN) ;
  uv=mxGetPr(UV_IN) ;
  T=*(mxGetPr(T_IN)) ;
  nthetav=mxGetM(THETAV_IN)*mxGetN(THETAV_IN) ; 
  nuv=mxGetM(UV_IN)*mxGetN(UV_IN) ; 
  nc=nthetav*nuv+3 ;

  if ((A_OUT=mxCreateDoubleMatrix(nc,nc,mxREAL))==NULL) 
    mexErrMsgTxt(NAME " Could not create array.") ;
  A=mxGetPr(A_OUT) ;

  if (bpmatfdi(thetav,nthetav,uv,nuv,T,A,nc))
    myErrMsg(NAME "Error in bpmatfdi") ;
}

double intu1(double a,double b,double p)
{ /* Evaluate the primitive function of u(x,ax+b) at x=p */
  double p2,b2,a2,p4,b4,b3,a4,p3 ;

  /* mexPrintf("intu1mex: a=%g b=%g p=%g\n",a,b,p) ; */
  b2=b*b ; b4=b2*b2 ; a2=a*a ; 
  if (p!=0.0) {
    p2=p*p ; p4=p2*p2 ; a4=a2*a2 ; 
    b3=b2*b ; p3=p2*p ;
    return (-24*p4*atan(a+b/p)+(-24*b4*atan((p+a*(b+a*p))/b)-
      6*a*(3+a2)*b4*log(b2+2*a*b*p+(1+a2)*p2)+
      (1+a2)*p*(12*(2+a2)*b3+18*a*(1+a2)*b2*p+4*(2+5*a2+3*a4)*b*p2+
      3*a*(3+4*a2+a4)*p3-6*(1+a2)*(4*b3+6*a*b2*p+4*(1+a2)*b*p2+
      a*(3+a2)*p3)*log(p2+(b+a*p)*(b+a*p))))/(1+a2)/(1+a2))/576/M_PI ;
  }
  return ((24*b4*atan(1/a))-6*a*(3+a2)*b4*log(b2))/(1+a2)/(1+a2)/576/M_PI ;
}

double intu2(double c,double p)
{ /* % Evaluate the primitive function of u(c,x) at x=p */
  double p2,p4,c2,c4 ;

  /* mexPrintf("intu2mex: c=%g p=%g\n",c,p) ; */

  p2=p*p ; p4=p2*p2 ; c2=c*c ; c4=c2*c2 ;
  if (c!=0.0) return (10*c2*p2+p4-32*c2*c*p*atan(p/c)-
      2*(-3*c4+6*c2*p2+p4)*log(c2+p2))/192/M_PI ;

  if (p!=0.0) return (p4-2*p4*log(p2))/192/M_PI ;
  return p4/192/M_PI ;
}

static double intv1(double a,double b,double p,double t,double T)
{
  double st=sqrt(T*T-t*t) ;
  /* mexPrintf("intv1mex: a=%g b=%g p=%g t=%g T=%g\n",a,b,p,t,T) ; */

  return intu1(a,a*t+b+st,p-t)/2-intu1(a,a*t+b-st,p-t)/2 ;
}

static double intv2(double c,double p,double t,double T)
{ /* Evaluate primitive function to v(c,x) at point x=p */
  double st=sqrt(T*T-t*t) ;
  /* mexPrintf("intv2mex: c=%g p=%g t=%g T=%g\n",c,p,t,T) ; */

  return intu2(c-t,p+st)/2-intu2(c-t,p-st)/2 ;
}



static double intI(double thdif,double q,double t,double T,double p)
{ double sint,cost,alpha0,alpha1,alpha2,alpha3 ;

  /* mexPrintf("intImex: thdif=%g q=%g t=%g T=%g p=%g\n",thdif,q,t,T,p) ; */
  sint=sin(thdif) ; cost=cos(thdif) ;
  alpha2=cost ; alpha3=-q*sint ; /* I change here x and y */
  alpha0=sint ; alpha1=q*cost ;

  if (alpha0!=0) 
    return intv1(alpha2/alpha0,-alpha2*alpha1/alpha0+alpha3,
		 p*alpha0+alpha1,t,T)/alpha0 ;
  return intv2(alpha1,p*alpha2+alpha3,t,T)/alpha2 ;
}


int bpmatfdi(double *thetav,int nthetav,double *uv,int nuv,double T,
	      double *A,int nc)

{ int iu,it,ju,jt,i,j ;
  double limT,val,val1,valx,valy,thdif,sint,cost,alpha,beta ;

  for (it=0;it<nthetav;it++)
    for (iu=0;iu<nuv;iu++) { /* sampling operator - rows */
      i=iu+nuv*it ;
      limT=T*T-uv[iu]*uv[iu] ;
      limT=limT>0 ? sqrt(limT) : 0 ;
      for (jt=0;jt<nthetav;jt++)
	for (ju=0;ju<nuv;ju++) { /* basis function - columns */
	  j=ju+nuv*jt ;
	  thdif=thetav[it]-thetav[jt] ;
	  val=intI(thdif,uv[iu],uv[ju],T,limT)-
	         intI(thdif,uv[iu],uv[ju],T,-limT) ;
	  A[i+j*nc]=val ;
	} /* end of for ju, jt */
      val1=2*limT ;
      sint=sin(thetav[it]) ; cost=cos(thetav[it]) ;
      valx=-2*limT*sint*uv[iu] ;
      valy=+2*limT*cost*uv[iu] ;
      A[i+nc*(nc-3)]=val1 ;  A[nc-3+i*nc]=val1 ;
      A[i+nc*(nc-2)]=valx ;  A[nc-2+i*nc]=valx ;
      A[i+nc*(nc-1)]=valy ;  A[nc-1+i*nc]=valy ;
    } /* end of for iu,it */
  return 0 ;
}
