/*
  mexbpmatxthr

  Generates the system matrix for a reconstruction using
  a linear combination of turned and shifted |x|^3 
  functions. Point (0,0) is in the middle of the 
  image, pixels correspond to step 1.

  Usage: A=mexbpmatxthr(thetav,uv,T) ;

  A...system matrix to find coefficients c=A\r
  thetav...projection angles in radians
  uv...displacements for measures for each angle in pixel units
  T...diametr of the circle inside which we integrate

  $Id: mexbpmatxthr.c,v 1.2 2000/04/12 18:10:18 jkybic Exp $

  Jan Kybic, 2000
*/

#include <math.h>
#include "mex.h"

/* define a few macros to keep the other source files generic */
#define myErrMsg mexErrMsgTxt
#define NAME "mexbpmatxthr"
#define myMalloc(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(double)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myMallocInt(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(int)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myFree(x) mxFree(x)


int bpmatxthr(double *thetav,int nthetav,double *uv,int nuv,double T,
	      double *A,int nc) ;

/* Input Arguments */

#define	THETAV_IN	   prhs[0]
#define	UV_IN              prhs[1]
#define	T_IN	           prhs[2]

/* Output Arguments */

#define	A_OUT	   plhs[0]

void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{ double *thetav, *uv, T, *A ;
  int i,nthetav,nuv,nc ;

  if (nrhs != 3) { /* Check for proper number of arguments */
    mexErrMsgTxt(NAME " requires three input arguments.");
  } else if (nlhs != 1) {
    mexErrMsgTxt(NAME " requires one output argument.");
  }

  for (i=0;i<nrhs;i++) /* Check for the type */
    { if (!mxIsNumeric(prhs[i]) || mxIsComplex(prhs[i]) || 
      mxIsSparse(prhs[i])  || !mxIsDouble(prhs[i])) 
         mexErrMsgTxt(NAME " needs all input arguments to be real.");
     } 
  
  thetav=mxGetPr(THETAV_IN) ;
  uv=mxGetPr(UV_IN) ;
  T=*(mxGetPr(T_IN)) ;
  nthetav=mxGetM(THETAV_IN)*mxGetN(THETAV_IN) ; 
  nuv=mxGetM(UV_IN)*mxGetN(UV_IN) ; 
  nc=nthetav*nuv+3 ;

  if ((A_OUT=mxCreateDoubleMatrix(nc,nc,mxREAL))==NULL) 
    mexErrMsgTxt(NAME " Could not create array.") ;
  A=mxGetPr(A_OUT) ;

  if (bpmatxthr(thetav,nthetav,uv,nuv,T,A,nc))
    myErrMsg(NAME "Error in bpmatxthr") ;
}

int bpmatxthr(double *thetav,int nthetav,double *uv,int nuv,double T,
	      double *A,int nc)

{ int iu,it,ju,jt,i,j ;
  double limT,val,val1,valx,valy,thdif,sint,cost,alpha,beta ;
  double tmp1, tmp2 ;
  double fzero=1e-10 ;

  for (it=0;it<nthetav;it++)
    for (iu=0;iu<nuv;iu++) { /* sampling operator - rows */
      i=iu+nuv*it ;
      limT=T*T-uv[iu]*uv[iu] ;
      limT=limT>0 ? sqrt(limT) : 0 ;
      for (jt=0;jt<nthetav;jt++)
	for (ju=0;ju<nuv;ju++) { /* basis function - columns */
	  j=ju+nuv*jt ;
	  thdif=thetav[jt]-thetav[it] ;
	  sint=sin(thdif) ; cost=cos(thdif) ;
	  alpha=fabs(sint) ; beta=cost*uv[iu]-uv[ju] ;
	  if (alpha<=fzero) {
	    val=fabs(beta)*beta*beta ;
	    val=limT*val/6 ;
	  } else if (-alpha*limT+beta>0) 
	    val=(limT*beta*(alpha*alpha*limT*limT+beta*beta))/6 ;
	  else if (alpha*limT+beta<0)
	    val=-(limT*beta*(alpha*alpha*limT*limT+beta*beta))/6 ;
	  else {
	    tmp1=alpha*limT+beta ; tmp2=-alpha*limT+beta ;
	    tmp1=tmp1*tmp1 ; tmp1=tmp1*tmp1 ;
	    tmp2=tmp2*tmp2 ; tmp2=tmp2*tmp2 ;
	    val=(tmp1+tmp2)/alpha/48 ;
	  }
	  A[i+j*nc]=val ;
	} /* end of for ju, jt */
      val1=2*limT ;
      sint=sin(thetav[it]) ; cost=cos(thetav[it]) ;
      valx=-2*limT*sint*uv[iu] ;
      valy=+2*limT*cost*uv[iu] ;
      A[i+nc*(nc-3)]=val1 ;  A[nc-3+i*nc]=val1 ;
      A[i+nc*(nc-2)]=valx ;  A[nc-2+i*nc]=valx ;
      A[i+nc*(nc-1)]=valy ;  A[nc-1+i*nc]=valy ;
    } /* end of for iu,it */
  return 0 ;
}
