/*
  mexbpmatzspln

  Generates the system matrix for a reconstruction using
  a linear combination of turned and shifted constant splines
  Point (0,0) is in the middle of the 
  image, pixels correspond to step 1.

  Usage: A=mexbpmatxthr(thetav,nuh,du,T) ;

  A...system matrix to find coefficients c=A\r
  thetav...projection angles in radians
  uv...displacements for measures for each angle in pixel units
  T...diametr of the circle inside which we integrate

  $Id: mexbpmatzspln.c,v 1.1 2000/05/04 08:36:18 jkybic Exp $

  Jan Kybic, 2000
*/

#include <math.h>
#include "mex.h"

/* define a few macros to keep the other source files generic */
#define myErrMsg mexErrMsgTxt
#define NAME "mexbpmatzspln"
#define myMalloc(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(double)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myMallocInt(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(int)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myFree(x) mxFree(x)


int bpmatzspln(double *thetav,int nthetav,int nuh,double du,double T,
	      double *A,int nc) ;

/* Input Arguments */

#define	THETAV_IN	   prhs[0]
#define	NUH_IN             prhs[1]
#define DU_IN              prhs[2]
#define	T_IN	           prhs[3]

/* Output Arguments */

#define	A_OUT	   plhs[0]

void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{ double *thetav, du, T, *A ;
  int i,nthetav,nc,nuh,nuv ;

  if (nrhs != 4) { /* Check for proper number of arguments */
    mexErrMsgTxt(NAME " requires four input arguments.");
  } else if (nlhs != 1) {
    mexErrMsgTxt(NAME " requires one output argument.");
  }

  for (i=0;i<nrhs;i++) /* Check for the type */
    { if (!mxIsNumeric(prhs[i]) || mxIsComplex(prhs[i]) || 
      mxIsSparse(prhs[i])  || !mxIsDouble(prhs[i])) 
         mexErrMsgTxt(NAME " needs all input arguments to be real.");
     } 
  
  thetav=mxGetPr(THETAV_IN) ;
  du=*(mxGetPr(DU_IN)) ;
  nuh=(int)(*(mxGetPr(NUH_IN))) ;
  T=*(mxGetPr(T_IN)) ;
  nthetav=mxGetM(THETAV_IN)*mxGetN(THETAV_IN) ; 
  nuv=2*nuh-1 ; 
  nc=nthetav*nuv ;

  if ((A_OUT=mxCreateDoubleMatrix(nc,nc,mxREAL))==NULL) 
    mexErrMsgTxt(NAME " Could not create array.") ;
  A=mxGetPr(A_OUT) ;

  if (bpmatzspln(thetav,nthetav,nuh,du,T,A,nc))
    myErrMsg(NAME "Error in bpmatxthr") ;
}

double intsec(double sint, double cost, double v, double u, double T)
{ double fzero=1e-10 ; double val ;

  if (sint<fzero) return u<v ? 0.0 : 2.0*T ; 
  val=T+(u*cost-v)/sint ;
  if (val<0.0) return 0.0 ;
  if (val>2.0*T) return 2.0*T ;
  return val ;
}

int bpmatzspln(double *thetav,int nthetav,int nuh,double du,double T,
	      double *A,int nc)

{ int iu,it,ju,jt,i,j ;
  double limT,val,thdif,sint,cost,alpha,beta ;
  double iuv,juv ;
  int nuv ;

  nuv=2*nuh-1 ;
  for (it=0;it<nthetav;it++)
    for (iu=0;iu<nuv;iu++) { /* sampling operator - rows */
      i=iu+nuv*it ;
      iuv=(1+iu-nuh)*du ;
      limT=T*T-iuv*iuv ;
      limT=limT>0 ? sqrt(limT) : 0 ;
      for (jt=0;jt<nthetav;jt++)
	for (ju=0;ju<nuv;ju++) { /* basis function - columns */
	  j=ju+nuv*jt ;
	  thdif=fabs(thetav[jt]-thetav[it]) ;
	  sint=sin(thdif) ; cost=cos(thdif) ;
	  juv=(1+ju-nuh)*du ;
	  val=intsec(sint,cost,juv-du/2,iuv,limT) ;
	  val-=intsec(sint,cost,juv+du/2,iuv,limT) ;
	  A[i+j*nc]=val ;
	} /* end of for ju, jt */
    } /* end of for iu,it */
  return 0 ;
}
