/*
  mexintgrxthr

  Evaluates an integral over a line (x1,y1)->(x2,y2) of an image
  described by a linear combination of turned and shifted |x|^3 
  functions.
  All coordinates use Matlab convention (start at 1, y,x) 
  Values outside of the image are assumed to be zero.

  Usage: val=mexintgrxthr(v,thetav,uv,c,tol) ;

  v=[y1 x1 y2 x2]...the integration path
  thetav...projection angles in radians
  uv...displacements for measures for each angle in pixel units
  c...coefficients - there should be Ntheta*Nuv+3 of them

  $Id: mexintgrxthr.c,v 1.4 2000/04/12 18:10:18 jkybic Exp $

  Jan Kybic, April 2000
*/

#include <math.h>
#include "mex.h"

/* define a few macros to keep the other source files generic */
#define myErrMsg mexErrMsgTxt
#define NAME "mexintgrimg"
#define myMalloc(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(double)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myMallocInt(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(int)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myFree(x) mxFree(x)


double intgrxthr(double y1,double x1, double y2, double x2, 
		double *thetav, int nthetav, double *uv, int nuv,
                double *c, int nc, double tol) ;

/* Input Arguments */

#define	V_IN	   prhs[0]
#define THETAV_IN  prhs[1]
#define UV_IN      prhs[2]
#define C_IN       prhs[3]
#define	TOL_IN    prhs[4]

/* Output Arguments */

#define	VAL_OUT	   plhs[0]

void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{ double *v, *img, *val ;
  double *thetav, *uv, *c,tol ;
  int nthetav,nuv,nc ;
  int i,dimy,dimx ;

  if (nrhs != 5) { /* Check for proper number of arguments */
    mexErrMsgTxt(NAME " requires five input arguments.");
  } else if (nlhs != 1) {
    mexErrMsgTxt(NAME " requires one output argument.");
  }

  for (i=0;i<nrhs;i++) /* Check for the type */
    { if (!mxIsNumeric(prhs[i]) || mxIsComplex(prhs[i]) || 
      mxIsSparse(prhs[i])  || !mxIsDouble(prhs[i])) 
         mexErrMsgTxt(NAME " needs all input arguments to be real.");
     } 
  
  v=mxGetPr(V_IN) ;
  if (mxGetM(V_IN)*mxGetN(V_IN)!=4) 
    mexErrMsgTxt(NAME ": input vector v should have length 4") ;
  thetav=mxGetPr(THETAV_IN) ;
  uv=mxGetPr(UV_IN) ;
  c=mxGetPr(C_IN) ;
  nthetav=mxGetM(THETAV_IN)*mxGetN(THETAV_IN) ; 
  nuv=mxGetM(UV_IN)*mxGetN(UV_IN) ; 
  nc=mxGetM(C_IN)*mxGetN(C_IN) ; 
  tol=*(mxGetPr(TOL_IN)) ;

  if ((VAL_OUT=mxCreateDoubleMatrix(1,1,mxREAL))==NULL) 
    mexErrMsgTxt(NAME " Could not create array.") ;
  val=mxGetPr(VAL_OUT) ;

  /* mexPrintf("Calling bpevalthrimg with nthetav=%d nuv=%d nc=%d dimx=%d dimy=%d\n", nthetav,nuv,nc,dimx,dimy) ;
   */
  *val=intgrxthr(v[0],v[1],v[2],v[3],thetav,nthetav,uv,nuv,c,nc,tol) ;
}

double fval(double t,double y1,double x1, double y2, double x2,
            double *thetav, int nthetav, double *uv, int nuv,
            double *c, int nc) 
{ double xa,ya,val ;
  double theta,sint,cost,tmp ;
  int it, iu ; 

  xa=x1+t*(x2-x1) ; ya=y1+t*(y2-y1) ;
  val=c[nc-3]+c[nc-2]*xa+c[nc-1]*ya ; 
  for (it=0;it<nthetav;it++) {
	theta=thetav[it] ; sint=sin(theta) ; cost=cos(theta) ;
	for (iu=0;iu<nuv;iu++) {
	  tmp=fabs(-sint*xa+cost*ya-uv[iu]) ;
	  val+=c[iu+it*nuv]*fabs(tmp*tmp*tmp)/12 ;
	}
  }
 finish:
  /* mexPrintf("fval xa=%g ya=%g t=%g gives val=%g\n",xa,ya,t,val) ; */
  return val ;
}
  

double rawintgrxthr(double y1,double x1, double y2, double x2, 
		double *thetav, int nthetav, double *uv, int nuv,
                double *c, int nc, double t1,double t2,int steps) 
{ double ll,val,xd,yd,td ;
  int t ;
  /* steps should be even */

  xd=x2-x1 ; yd=y2-y1 ; td=(t2-t1)/steps ;
  ll=sqrt(xd*xd+yd*yd) ;
  val=0.0 ; 
  val+=fval(t1,y1,x1,y2,x2,thetav,nthetav,uv,nuv,c,nc) ;
  val+=4*fval(t2-td*0.5,y1,x1,y2,x2,thetav,nthetav,uv,nuv,c,nc) ;
  val+=fval(t2,y1,x1,y2,x2,thetav,nthetav,uv,nuv,c,nc) ;

  for (t=1;t<steps;t++) {
    val+=4*fval(t1+((double)t-0.5)*td,y1,x1,y2,x2,thetav,nthetav,uv,nuv,c,nc) ;
    val+=2*fval(t1+((double)t)*td,y1,x1,y2,x2,thetav,nthetav,uv,nuv,c,nc) ;
  }
  val*=((double)ll)*td/6.0 ;
  /*  mexPrintf("Calling rawintgrxthr t1=%g t2=%g td=%g steps=%d gives %g\n",
	    t1,t2,td,steps,val) ;
  */
  return val ;
}

double adaptintgrxthr(double y1,double x1, double y2, double x2, 
		double *thetav, int nthetav, double *uv, int nuv,
                double *c, int nc,double t1,double t2, double tol) 
{ double val,tm ;
 
  val=rawintgrxthr(y1,x1,y2,x2,thetav,nthetav,uv,nuv,c,nc,t1,t2,12) ;
  if (fabs(val-
	    rawintgrxthr(y1,x1,y2,x2,thetav,nthetav,uv,nuv,c,nc,t1,t2,6))<tol)
     return val ;
  tm=(t1+t2)/2 ;   
  return   adaptintgrxthr(y1,x1,y2,x2,thetav,nthetav,uv,nuv,c,nc,t1,tm,tol/2)+
  adaptintgrxthr(y1,x1,y2,x2,thetav,nthetav,uv,nuv,c,nc,tm,t2,tol/2) ;

}


double intgrxthr(double y1,double x1, double y2, double x2, 
		double *thetav, int nthetav, double *uv, int nuv,
                double *c, int nc, double tol) 
{ 
   return adaptintgrxthr(y1,x1,y2,x2,thetav,nthetav,uv,nuv,c,nc,0.0,1.0,tol) ;
}


