/*		mex_filiirs.c	: Matlab .MEX file
 
 M. Unser / AUG-93 

  The matlab calling syntax is:

			[y] = filiirs(x, zi, c0)

IIR filter, see comments below. Matlab interface is in fbspline.m

Modified by J.K. April 1998, to compile under Matlab 5 and gcc
$Id: filiirs.c,v 1.4 2000/01/31 16:30:41 jkybic Exp $

*/

#include <math.h>
#include "mex.h"

/*************************************************************
 *	Title : filiirs.c
 *
 * PURPOSE : Symmetric infinite impulse response filter
 * of the form :
 *		H[z] = c0 * Product[ -zi[i] / ( (1-zi[i]*z) (1-zi[i]/z) ), {i,1,nz}]
 *
 * Signals are extended using mirror boundary conditions.
 *
 *  Michael Unser / BEIP	JAN-90 / AUG-93
 *
 * Input signal  : x[k], k=0,....,nx-1
 * Output signal : y[k], k=0,....,nx-1
 *
 * Note : For a unity gain, c0 should be set to
 * c0= Product[(1-zi[i])^2/-zi[i],{i,1,nz}]
 *************************************************************/
#define MIN(x,y) ( ( (x) < (y) ) ? (x) : (y) )

void filiirs(double x[],double y[],int nx,double zi[],int nz,double c0);
void filiirs(double x[],double y[],int nx,double zi[],int nz,double c0)
{
  double x0,xN,log10_epsi=-8.,xk;
  int k,iz,k0;
  for (k=0;k<nx;k++) y[k]=c0*x[k];
  for (iz=0;iz<nz;iz++)
    {
      k0=(int)(log10_epsi/log10(fabs(zi[iz])));
      k0=MIN(k0,nx-1);
      //  mex_printf("k0=%d" ,(INT)k0);
      x0=y[k0];xN=y[nx-1];

      // Here is a problem if k0>nr
      for (k=k0-1;k>=0;k--) x0=y[k]+zi[iz]*x0;
      y[0]=x0;
      for (k=1;k<nx;k++) y[k]=y[k]+zi[iz]*y[k-1];
      y[nx-1]=-zi[iz]*(2.*y[nx-1] - xN) / (1.-zi[iz]*zi[iz]);
      for (k=nx-2;k>=0;k--) y[k]=zi[iz]*(y[k+1]-y[k]);
    }

}

/************************************************************************/
/* +=============================+ */
/* | FUNCTION: iirConvolveMirror |*/
/* +=============================+*/
/*----------------------------------------------------------------------*/
/**/
/* Objective:	Discrete recursive convolution*/
/**/
/* Conventions:	The kernel has a size np*/
/*The kernel is given by its (real) poles*/
/*The signal has an infinite, periodic (2 n - 1) support	*/
/*The input is described by n values; the missing values	*/
/*are provided by mirror symetry*/
/*The output has same length n than the input*/
/*The output has already been allocated					*/
/**/
/* Comment:		The output may share data storage with the input*/
/**/
/************************************************************************/
void				iirConvolveMirror
(double				*inPtr,
 double				*outPtr,
 long				n,
 double				gain,
 double				*realPoles,
 long				np) 

{
  
  double *p, *q, *r;
  double tolerance = log10((double)FLT_EPSILON);
  double x0, pole;
  long n2;
  long i, j, k;
  
  p = inPtr;
  q = outPtr;
  r = outPtr + (ptrdiff_t)n;
  while (q < r)
    *q++ = *p++ * gain;
  if (n == 1L) {
    p = realPoles;
    q = realPoles + (ptrdiff_t)np;
    while (p <= --q)
      *outPtr /= (1.0 - *q) * (1.0 - *q);
    return;
  }
  n2 = 2L * (n - 1L);
  for (i = -np; (i < 0L); i++) {
    pole = *realPoles++;
    j = (long)ceil(tolerance / log10(fabs(pole)));
    k = j - n2 * (j / n2);
    j -= k;
    if (k < n) {
      p = outPtr + (ptrdiff_t)k;
      x0 = *p;
    }
    else {
      k = n2 - k;
      p = outPtr + (ptrdiff_t)k;
      x0 = *p;
      while (++p < r)
	x0 = pole * x0 + *p;
      p--;
    }
    while (--p >= outPtr)
      x0 = pole * x0 + *p;
    while (j > 0L) {
      p++;
      while (++p < r)
	x0 = pole * x0 + *p;
      p--;
      while (--p >= outPtr)
	x0 = pole * x0 + *p;
      j -= n2;
    }
    q = p++;
    *p++ = x0;
    x0 = *(q++ + (ptrdiff_t)n);
    while (p < r)
      *p++ += *q++ * pole;
    *q = (2.0 * *q - x0) / (1.0 - pole * pole);
    p--;
    while (--q >= outPtr)
      *q += *p-- * pole;
  }
} /* End of iirConvolveMirror */


/*--------------------------------------------------------------------------*/
enum TBoundaryConvention
{
  AntiMirrorOnBounds,
  FiniteCoefficientSupport,
  FiniteDataSupport,
  MirrorOffBounds,
  MirrorOnBounds,
  Periodic
};

#define ERROR (1)
#define WRITE_ERROR(x,s) mexPrintf(s "\n") ;

double PositiveIntPower(double x,int n)
{
  double y=1.0 ;
  while (n--) y*=x ;
  return y;
}

long ConvertDoubleToLong(double x) { return (long)x ; }

extern int		IirConvolvePoles
(
 double	InputData[],		/* data to process */
 double	OutputData[],		/* result */
 long	SignalLength,		/* length of the 1D data array */
 double	RealPoles[],		/* array of real poles */
 long	PoleNumber,		/* number of poles */
 enum TBoundaryConvention
 Convention,			/* boundary convention */
 double	Tolerance		/* admissible relative error */
 );

extern int		IirConvolvePoles
(
 double	InputData[],		/* data to process */
 double	OutputData[],		/* result */
 long	SignalLength,		/* length of the 1D data array */
 double	RealPoles[],		/* array of real poles */
 long	PoleNumber,			/* number of poles */
 enum TBoundaryConvention
 Convention,			/* boundary convention */
 double	Tolerance			/* admissible relative error */
 )

     /* the input and the output have the same length */
     /* in-place processing is allowed */
     /* no more than two poles are allowed for a finite support boundary */
     /* success: return(!ERROR); failure: return(ERROR) */

{ /* begin IirConvolvePoles */

  double	*p, *q;
  double	Sum, Gain;
  double	z, z0, z1, z2, iz;
  long	Horizon;
  long	i, j;
  int		Status = !ERROR;
  
  Gain = 1.0;
  for (i = 0L; (i < PoleNumber); i++) {
    z = RealPoles[i];
    if (0.0 <= z) {
      Status = ERROR;
      WRITE_ERROR(IirConvolvePoles,
		  "Invalid pole (should be strictly negative)")
	return(Status);
    }
    z1 = 1.0 - z;
    Gain *= -(z1 * z1) / z;
  }
  p = InputData;
  q = OutputData;
  for (i = -SignalLength; (i < 0L); i++) {
    *q++ = *p++ * Gain;
  }
  switch (Convention) {
  case AntiMirrorOnBounds:
    if (SignalLength == 1L) {
      *OutputData = *InputData;
      return(Status);
    }
    for (i = 0L; (i < PoleNumber); i++) {
      z = RealPoles[i];
      Sum = ((1.0 + z) / (1.0 - z)) * (OutputData[0]
				       - PositiveIntPower(z, SignalLength - 1L) 
				       * OutputData[SignalLength - 1L]);
      z1 = z;
      z2 = PositiveIntPower(z, 2L * SignalLength - 3L);
      iz = 1.0 / z;
      for (j = 1L; (j < (SignalLength - 1L)); j++) {
	Sum += (z2 - z1) * OutputData[j];
	z1 *= z;
	z2 *= iz;
      }
      OutputData[0] = Sum / (1.0 - 
			     PositiveIntPower(z, 2L * SignalLength - 2L));
      for (j = 1L; (j < SignalLength); j++) {
	OutputData[j] += z * OutputData[j - 1L];
      }
      OutputData[SignalLength - 1L] = (-z / ((1.0 - z) * (1.0 - z)))
	* (OutputData[SignalLength - 1L] 
	   - z * OutputData[SignalLength - 2L]);
      for (j = SignalLength - 2L; (0L <= j); j--) {
	OutputData[j] = z * (OutputData[j + 1L] - OutputData[j]);
      }
    }
    break;
  case FiniteCoefficientSupport:
    switch (PoleNumber) {
    case 1L:
      z = RealPoles[0];
      Sum = (OutputData[0] - PositiveIntPower(z, SignalLength + 1L)
	     * OutputData[SignalLength - 1L]) * (1.0 + z) / (z * z);
      z1 = z;
      z2 = PositiveIntPower(z, 2L * SignalLength - 2L);
      iz = 1.0 / z;
      for (j = 1L; (j < (SignalLength - 1L)); j++) {
	Sum -= (z2 + z1) * OutputData[j];
	z1 *= z;
	z2 *= iz;
      }
      OutputData[0] = Sum / ((1.0 + z) * ((1.0 + z * z
         + PositiveIntPower(z, 2L * SignalLength)) / (z * z)
         + (z * z - PositiveIntPower(z, SignalLength)) / (1.0 - z)));
      for (j = 1L; (j < SignalLength); j++) {
	OutputData[j] += z * OutputData[j - 1L];
      }
      OutputData[SignalLength - 1L] *= -z;
      for (j = SignalLength - 2L; (0L <= j); j--) {
	OutputData[j] = z * (OutputData[j + 1L] - OutputData[j]);
      }
      break;
    default:
      Status = ERROR;
      WRITE_ERROR(IirConvolvePoles,
        "Invalid number of poles (should be 1 for FiniteCoefficientSupport)")
	break;
    }
    break;
  case FiniteDataSupport:
    switch (PoleNumber) {
    case 1L:
      z = RealPoles[0];
      for (j = 1L; (j < SignalLength); j++) {
	OutputData[j] += z * OutputData[j - 1L];
      }
      OutputData[SignalLength - 1L] *= z / (z * z - 1.0);
      for (j = SignalLength - 2L; (0L <= j); j--) {
	OutputData[j] = z * (OutputData[j + 1L] - OutputData[j]);
      }
      break;
    case 2L:
      z0 = RealPoles[0];
      for (j = 1L; (j < SignalLength); j++) {
	OutputData[j] += z0 * OutputData[j - 1L];
      }
      OutputData[SignalLength - 1L] *= z0 / (z0 * z0 - 1.0);
      z = OutputData[SignalLength - 1L];
      for (j = SignalLength - 2L; (0L <= j); j--) {
	OutputData[j] = z0 * (OutputData[j + 1L] - OutputData[j]);
      }
      z1 = RealPoles[1];
      OutputData[0] /= 1.0 - z0 * z1;
      for (j = 1L; (j < SignalLength); j++) {
	OutputData[j] += z1 * OutputData[j - 1L];
      }
      OutputData[SignalLength - 1L] = (z1 / (z1 * z1 - 1.0))
	* (OutputData[SignalLength - 1L] - (z0 * z1 / (z0 * z1 - 1.0)) * z);
      for (j = SignalLength - 2L; (0L <= j); j--) {
	OutputData[j] = z1 * (OutputData[j + 1L] - OutputData[j]);
      }
      break;
    default:
      Status = ERROR;
      WRITE_ERROR(IirConvolvePoles,
       "Invalid number of poles (should be 1 or 2 for FiniteDataSupport)")
	break;
    }
    break;
  case MirrorOffBounds:
    if (SignalLength == 1L) {
      *OutputData = *InputData;
      return(Status);
    }
    for (i = 0L; (i < PoleNumber); i++) {
      z = RealPoles[i];
      Sum = (OutputData[0] + PositiveIntPower(z, SignalLength)
	     * OutputData[SignalLength - 1L]) * (1.0 + z) / z;
      z1 = z;
      z2 = PositiveIntPower(z, 2L * SignalLength - 2L);
      iz = 1.0 / z;
      for (j = 1L; (j < (SignalLength - 1L)); j++) {
	Sum += (z2 + z1) * OutputData[j];
	z1 *= z;
	z2 *= iz;
      }
      OutputData[0] = Sum * z / (1.0 - PositiveIntPower(z, 2L * SignalLength));
      for (j = 1L; (j < SignalLength); j++) {
	OutputData[j] += z * OutputData[j - 1L];
      }
      OutputData[SignalLength - 1L] *= z / (z - 1.0);
      for (j = SignalLength - 2L; (0L <= j); j--) {
	OutputData[j] = z * (OutputData[j + 1L] - OutputData[j]);
      }
    }
    break;
  case MirrorOnBounds:
    if (SignalLength == 1L) { *OutputData = *InputData;
			      return(Status);
    }
    for (i = 0L; (i < PoleNumber); i++) {
      z = *RealPoles++;
      if (Tolerance == 0.0) {
	Horizon = SignalLength;
      }
      else {
	Horizon = ConvertDoubleToLong(ceil(log(Tolerance) / log(fabs(z))));
      }
      if (Horizon < SignalLength) {
	z1 = z;
	Sum = *OutputData;
	p = OutputData;
	for (j = -Horizon; (j < 0L); j++) {
	  Sum += z1 * *++p;
	  z1 *= z;
	}
	p = OutputData;
	q = OutputData;
	*p++ = Sum;
	for (j = 1L - SignalLength; (j < 0L); j++) {
	  *p++ += z * *q++;
	}
	p = q--;
	*p = (z * *q + *p) * z / (z * z - 1.0);
	for (j = SignalLength - 2L; (0L <= j); j--) {
	  *q-- = z * (*p-- - *q);
	}
      }
      else {
	z1 = z;
	z2 = PositiveIntPower(z, SignalLength - 1L);
	iz = 1.0 / z;
	Sum = *OutputData + z2 * OutputData[SignalLength - 1L];
	z2 *= z2 * iz;
	p = OutputData;
	for (j = 2L - SignalLength; (j < 0L); j++) {
	  Sum += (z1 + z2) * *++p;
	  z1 *= z;
	  z2 *= iz;
	}
	p = OutputData;
	q = OutputData;
	*p++ = Sum / (1.0 - z2 * z2);
	for (j = 1L - SignalLength; (j < 0L); j++) {
	  *p++ += z * *q++;
	}
	p = q--;
	*p = (z * *q + *p) * z / (z * z - 1.0);
	for (j = SignalLength - 2L; (0L <= j); j--) {
	  *q-- = z * (*p-- - *q);
	}
      }
    }
    break;
  case Periodic:
    if (SignalLength == 1L) {
      *OutputData = *InputData;
      return(Status);
    }
    for (i = 0L; (i < PoleNumber); i++) {
      z = RealPoles[i];
      z2 = PositiveIntPower(z, SignalLength - 1L);
      iz = 1.0 / z;
      Sum = OutputData[0];
      for (j = 1L; (j < SignalLength); j++) {
	Sum += z2 * OutputData[j];
	z2 *= iz;
      }
      OutputData[0] = Sum / (1.0 - PositiveIntPower(z, SignalLength));
      for (j = 1L; (j < SignalLength); j++) {
	OutputData[j] += z * OutputData[j - 1L];
      }
      Sum = OutputData[0] + OutputData[SignalLength - 1L] / z;
      z1 = z;
      for (j = 1L; (j < (SignalLength - 1L)); j++) {
	Sum += z1 * OutputData[j];
	z1 *= z;
      }
      OutputData[SignalLength - 1L] = Sum * z * z
	/ (PositiveIntPower(z, SignalLength) - 1.0);
      for (j = SignalLength - 2L; (0L <= j); j--) {
	OutputData[j] = z * (OutputData[j + 1L] - OutputData[j]);
      }
    }
    break;
  default:
    Status = ERROR;
    WRITE_ERROR(IirConvolvePoles,"Invalid boundary convention")
      break;
  }
  return(Status);
} /* end IirConvolvePoles */


/*--------------------------------------------------------------------------*/




/* Input Arguments */

#define	X_IN	prhs[0]
#define	Z_IN	prhs[1]
#define	C0_IN	prhs[2]

/* Output Arguments */

#define	Y_OUT	plhs[0]


#define	max(A, B)	((A) > (B) ? (A) : (B))
#define	min(A, B)	((A) < (B) ? (A) : (B))


#ifdef __STDC__
void mexFunction(
	int		nlhs,
	mxArray	*plhs[],
	int		nrhs,
	const mxArray	*prhs[]
	)
#else
mexFunction(nlhs, plhs, nrhs, prhs)
int nlhs, nrhs;
mxArray *plhs[], *prhs[];
#endif
{
	double	*y;
	double	*x,*zi,c0;
	unsigned int	m,n;
	int nx,nz,nmin;


	/* Check for proper number of arguments */

	if (nrhs != 3) {
		mexErrMsgTxt("FILIIRS requires three input arguments.");
	} else if (nlhs != 1) {
		mexErrMsgTxt("FILIIRS requires one output argument.");
	}


	/* Check the dimensions of input variables */

	m = mxGetM(X_IN);
	n = mxGetN(X_IN);
	nx=max(m,n);nmin=min(m,n);

	if (!mxIsNumeric(X_IN) || mxIsComplex(X_IN) || 
		mxIsSparse(X_IN)  || !mxIsDouble(X_IN) ||
		(nmin != 1)) {
		mexErrMsgTxt("FILIIRS requires that X be a vector.");
	}

	/* Create a matrix for the return argument */

	Y_OUT = mxCreateDoubleMatrix(m,n,mxREAL) ;

	m = mxGetM(Z_IN);
	n = mxGetN(Z_IN);
	nz=max(m,n);nmin=min(m,n);

	if (!mxIsNumeric(Z_IN) || mxIsComplex(Z_IN) || 
		mxIsSparse(Z_IN)  || !mxIsDouble(Z_IN) ||
		(nmin != 1)) {
		mexErrMsgTxt("FILIIRS requires that ZI be a vector.");
	}

	/* Assign pointers to the various parameters */

	y = mxGetPr(Y_OUT);
	x = mxGetPr(X_IN);
	zi  = mxGetPr(Z_IN);
	c0= (double) *mxGetPr(C0_IN);
	
//	mex_printf("nx=%d, nz=%d, c0=%f",(INT) nx,(INT) nz, (DOUBLE) c0);

	/* Do the actual computations in a subroutine */
#if 0
	filiirs(x,y,nx,zi,nz,c0);
#endif
#if 1
	/* mexPrintf("iirConvolveMirror\n") ; */
	iirConvolveMirror(x,y,nx,c0,zi,nz) ; /* P.T. */
#endif
#if 1 
	IirConvolvePoles(x,y,nx,zi,nz,MirrorOnBounds,1e-20) ;
        for (m=0;m<nx;m++) *(y++)*=c0 ;
#endif
	return;
}

