function [c,T,d]=lsrsplnapr(x,y,N,deg,lambda,nder)
% approximates data x,y by a spline of degree deg described by N
% uniformly placed coefficients for B-splines. The approximation is
% f(x)=sum_{i=1}^N c_i beta_deg(x/T-i+d)
%
% The approximation minimizes
% J(f)=sum_{k=1}^M (f(x_k)-y_k)^2 + lambda \int (d^nderf/dx^nder)^2 dx
%
% Usage: [c,T,d]=lsrsplnapr(x,y,N,deg,lambda,nder)
% Jan Kybic, March 2000
% $Id: lsrsplnapr.m,v 1.9 2000/05/05 11:00:52 jkybic Exp $

% calculate T,d, so as to put enough knots outside the interval as not to
% impose any additional constraints

x=x(:) ; y=y(:) ;
xmin=min(x) ; xmax=max(x) ;
ofs=(deg-1)/2 ; 
T=(xmax-xmin)/(N-1-2*ofs) ; 
d=1+ofs-xmin/T ;

m=length(x) ;
% assemble system matrix
A=zeros(N,N) ; 
r=zeros(N,1) ; % right hand side 

bv=zeros(m,N) ; % precalculated spline values
for j=1:N,
  tj=x/T-j+d ;
  bv(:,j)=evalbspline(tj,deg)' ;
  r(j)=bv(:,j)'*y ;
end ;

[tmp,binc,shift]=evalbspline(0,deg) ;

for j=1:N, % for all unknows - lines
  for i=1:N, % for all unknowns - columns
    A(j,i)=bv(:,j)'*bv(:,i) ; 
  end ;  
end ;

Ar=zeros(N,N) ;
% add regularization
xmint=xmin/T+d ; 
xmaxt=xmax/T+d ;
%xmint=1 ; xmaxt=N ;

for j=1:N,
  for i=1:N,
    Ar(j,i)=regint(i,j,deg,binc,shift,xmint,xmaxt,nder) ;
  end ;
end ;


A=A+lambda*2*factorial(deg)/factorial(deg-nder)*m/(N-1)*Ar ; % 840=7*6*5*4

% solve the set
%[u,s,v]=svd(A) ;
%s=diag(s) ; q=(s(1)*1e-20+1e-50) ;
%s(1)/s(length(s))
%for i=1:length(s),
% if s(i)<q, s(i)=0 ; 
%  else s(i)=1/s(i) ; end ;
%end ;
%c=v*diag(s)*u'*r ;
c=A\r ;

disp([ 'Residual ' num2str(max(max(abs(r-A*c)))/max(abs(r))) ]) ;

function val=regint(i,j,deg,binc,shift,xmint,xmaxt,nder) ;
% Calculate integral between xmint and xmaxt from
% beta_deg(x-i)^(nder)*beta_deg(x-j)^(nder) 

val=0 ; n=deg+2 ;
for k=1:n,
    for l=1:n,
      val=val+binc(k)*binc(l)*...
         intktitp(i+shift(k),j+shift(l),xmint,xmaxt,deg-nder) ; 
    end ;
end ;

function val=intktitp(it,kt,xmin,xmax,n) ;
% Integrates (x-it)+^n*(x-kt)+^n from xmin to xmax
ll=max([ it kt xmin]) ;
if ll<xmax,
  val=intktit(it,kt,ll,xmax,n) ;
else 
  val=0 ;
end ;

function val=intktit(it,kt,xmin,xmax,n) ;
% Integrates (x-it)^n*(x-kt)^n from xmin to xmax
%
a=-(it+kt) ; b=it*kt ;
val=intxab(a,b,xmax,n)-intxab(a,b,xmin,n) ;

function val=intxab(a,b,x,n) ;
% Calculates the primitive function to (x^2+a x+b)^n at point x
if n==4,
 val=x*(b^4+x*(2*a*b^3+x*(2*a^2*b^2+4*b^3/3+x*(a^3*b+3*a*b^2+...
     x*(a^4/5+12*a^2*b/5+6*b^2/5+x*(2*a^3/3+2*a*b+...
     x*(6*a^2/7+4*b/7+(a/2+x/9)*x))))))) ; 
elseif n==3,
  val=x*(b^3+x*(1.5*a*b^2+x*(b*(a^2+b)+...
      x*(a/4*(a^2+6*b)+x*(0.6*(a^2+b)+x*(a/2+x/7)))))) ;
elseif n==2,
  val=x*(b^2+x*(a*b+x*((a^2+2*b)/3+x*(a/2+x/5)))) ;
elseif n==1,
  val=x*(6*b+x*(3*a+2*x))/6 ;
elseif n==0,  
  val=x ;
else
  error('Unsupported n in intxab(lsrsplnapr)') 
end ;

