/*
  mexsdcgr    The MEX source file

% Calculates the SSD criterion and its gradient
% fr - reference image, ftc - test image B-spline coefs
% c - deformation coefs. degft, degc - spline degrees
% H - Hessian, second derivatives of E with respect to c
%
% gr is always returned as a vector, H as a matrix, regardless of the input
% dimensionality
%
% Usage: [E,gr,H]=mexsdcgr(fr,ftc,c,degft,degc)
%
% Jan Kybic, 1999
% $Id: mexsdcgr.c,v 1.6 1999/11/23 13:01:01 jkybic Exp $

*/

#include "mex.h"

/* Input Arguments */

#define	FR_IN	  prhs[0]
#define	FTC_IN	  prhs[1]
#define	C_IN	  prhs[2]
#define	DEGFT_IN  prhs[3]
#define DEGC_IN   prhs[4]

/* Output Arguments */

#define	E_OUT	plhs[0]
#define	GR_OUT	plhs[1]
#define	H_OUT	plhs[2]

#define NAME "MEXSDCGR"
#define myErrMsg mexErrMsgTxt
#define myMalloc(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(double)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myMallocInt(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(int)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myFree(x) mxFree(x)

#define max(a,b) ((a>b) ? (a) : (b))

void sdcgr(double *fr,double *ftc,double *c,double *E,
		   double *gr,double *H,int degft, int degc, 
	   int frx,int fry,int frz,
           int ftx,int fty,int ftz,
           int cx,int cy,int cz,int nouts) ; 

void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{ double *c,*fr,*ftc,*E,*gr,*H ;
  int i,frx,fry,frz,ftx,fty,ftz,cx,cy,cz,degft,degc ;
  int ndim,cdim ; const int *dims ;

  /* Check for proper number of arguments */
  
  if (nrhs != 5) {
    mexErrMsgTxt(NAME " requires five input arguments.");
  } else if (nlhs > 3 || nlhs<1) {
    mexErrMsgTxt(NAME " requires one to three output argument.");
  }
  
  for (i=0;i<nrhs;i++)
    { if (!mxIsNumeric(prhs[i]) || mxIsComplex(prhs[i]) || 
      mxIsSparse(prhs[i])  || !mxIsDouble(prhs[i])) 
         mexErrMsgTxt(
             NAME " needs all input arguments to be real.");
     } 
    
  c=mxGetPr(C_IN) ; fr=mxGetPr(FR_IN) ; ftc=mxGetPr(FTC_IN) ;
  ndim=mxGetNumberOfDimensions(C_IN) ;
  dims=mxGetDimensions(C_IN) ;  
  if (ndim>4) mexErrMsgTxt(NAME " unsupported dimensionality of c.") ;
  // mexPrintf("ndim=%d c=(%d,%d,%d)\n",ndim,dims[0],dims[1],dims[2]) ;  
  cz=cy=1 ; cdim=-1 ;
  if (ndim==4)
    if (dims[3]>1) { cx=dims[1] ; cy=dims[2] ; cz=dims[3] ; cdim=3 ; }
         else ndim-- ;
  if (ndim==3)
    if (dims[2]>1) { cx=dims[1] ; cy=dims[2] ; cdim=2 ; }
         else ndim-- ;
  if (ndim==2 && (cx=max(dims[0],dims[1]))==dims[0]*dims[1]) cdim=1 ;
  if (ndim==1) { cx=dims[0] ; cdim=1 ; }
  if (cdim==-1) mexErrMsgTxt(NAME " unsupported dimensionality of c.") ;

  ndim=mxGetNumberOfDimensions(FR_IN) ;
  dims=mxGetDimensions(FR_IN) ;  
  if (ndim>3) mexErrMsgTxt(NAME " unsupported dimensionality of fr.") ;
  frx=dims[0] ; fry=dims[1] ; frz=ndim>2 ? dims[2] : 1 ;
  ndim=mxGetNumberOfDimensions(FR_IN) ;
  dims=mxGetDimensions(FR_IN) ;  
  if (ndim>3) mexErrMsgTxt(NAME " unsupported dimensionality of ftc.") ;
  ftx=dims[0] ; fty=dims[1] ; ftz=ndim>2 ? dims[2] : 1 ;

  degft=(int)(*(mxGetPr(DEGFT_IN))) ;
  degc=(int)(*(mxGetPr(DEGC_IN))) ;

  // mexPrintf("cdim=%d c=(%d,%d,%d)\n",cdim,cx,cy,cz) ; 

  E_OUT = mxCreateDoubleMatrix(1,1,mxREAL); E=mxGetPr(E_OUT) ;
  gr=NULL ; H=NULL ;
  if (nlhs>1) {
    GR_OUT = mxCreateDoubleMatrix(cdim*cx*cy*cz,1,mxREAL); 
    gr=mxGetPr(GR_OUT) ;
  }
  if (nlhs>2) {
    H_OUT = mxCreateDoubleMatrix(cdim*cx*cy*cz,cdim*cx*cy*cz,mxREAL); 
    H=mxGetPr(H_OUT) ;
  }

  /*  %%%%% here the calculation begins %%%%%% */
  sdcgr(fr,ftc,c,E,gr,H,degft,degc,frx,fry,frz,ftx,fty,ftz,cx,cy,cz,nlhs) ; 
}


#define MAKING_MEX
#include "sdcgr.c"

/* end of meximwarpcalc */
