/*
  mexsplninterp   Matlab interface to splninterp.c

  Takes a function described by B-spline coefficients of `degree', mirror 
  boundary conditions. Evaluates it at all points in the coord matrix, each 
  coordinate having dim components. Stores results in outp. If `dflag' is not 
  given or zero, evaluates the value. Otherwise:
  dflag=1,4,16   derivative with respect to x,y,z
  dflag=2,8,32   second derivative with respect to x,y,z

  Usage: outp=mexsplninterp(inp,coord,dim,degree,[dflag]) ;

  $Id: mexsplninterp.c,v 1.8 2000/08/03 13:25:24 jkybic Exp $

  Jan Kybic, 1999
*/

#include <math.h>
#include "mex.h"

/* define a few macros to keep the other source files generic */
#define myErrMsg mexErrMsgTxt
#define NAME "mexsplninterp"
#define myMalloc(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(double)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myMallocInt(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(int)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myFree(x) mxFree(x)

/* Including source files is not very clean, but works best for generating
   MEX files. Replace by linking for other applications */
#define MEX 1
#include "bsplneval.c"
#include "splninterp.c"


/* Input Arguments */

#define	INP_IN	   prhs[0]
#define	COORD_IN   prhs[1]
#define	DIM_IN	   prhs[2]
#define DEGREE_IN  prhs[3]
#define DFLAG_IN   prhs[4]

/* Output Arguments */

#define	OUTP_OUT	plhs[0]

void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{ double *inp, *coord,*outp ;
  int i,degree,nxi,nyi,nzi,nxc,nyc,nzc,ndc ;
  int ndim; const int *dims ;
  int dflag ;

  if (nrhs != 4 && nrhs!=5) { /* Check for proper number of arguments */
    mexErrMsgTxt(NAME " requires four or five input arguments.");
  } else if (nlhs != 1) {
    mexErrMsgTxt(NAME " requires one output argument.");
  }

  for (i=0;i<nrhs;i++) /* Check for the type */
    { if (!mxIsNumeric(prhs[i]) || mxIsComplex(prhs[i]) || 
      mxIsSparse(prhs[i])  || !mxIsDouble(prhs[i])) 
         mexErrMsgTxt(NAME " needs all input arguments to be real.");
     } 
  
  dflag= nrhs==4 ? 0 : (int)(*(mxGetPr(DFLAG_IN))) ;

  degree=(int)(*(mxGetPr(DEGREE_IN))) ;
  ndc=(int)(*(mxGetPr(DIM_IN))) ;
  coord=mxGetPr(COORD_IN) ;
  inp=mxGetPr(INP_IN) ;

  ndim=mxGetNumberOfDimensions(INP_IN) ;
  dims=mxGetDimensions(INP_IN);
  if (ndim>3) mexErrMsgTxt(NAME " unsupported dimensionality.") ;
  nxi=dims[0] ; /* the fastest changing index, whichever it is */
  nyi=dims[1] ; nzi=ndim<3 ? 1 : dims[2] ; 

  ndim=mxGetNumberOfDimensions(COORD_IN) ;
  dims=mxGetDimensions(COORD_IN);
  if (ndim>4) mexErrMsgTxt(NAME " unsupported dimensionality.") ;
  if (ndc==1) {
    nxc=dims[0] ; /* the fastest changing index, whichever it is */
    nyc=dims[1] ; nzc=ndim<3 ? 1 : dims[2] ; 
  } else {
    nxc=dims[1] ; /* the fastest changing index, whichever it is */
    nyc=ndim<3 ? 1 : dims[2] ; nzc=ndim<4 ? 1 : dims[3] ; 
  }

  if ((OUTP_OUT=
       mxCreateNumericArray(ndc==1 ? ndim : ndim-1,
       ndc==1 ? dims : dims+1,mxDOUBLE_CLASS,mxREAL))==NULL) 
    mexErrMsgTxt(NAME " Could not create array.") ;
  outp=mxGetPr(OUTP_OUT) ;
 if (splinterp(inp,coord,outp,nxi,nyi,nzi,nxc,nyc,nzc,ndc,degree,dflag))
    myErrMsg(NAME "Error in splinterp") ;
}
