/*
  mexsplninterpone   Matlab interface to splninterp.c

  Samples a B-spline of degree `degree', centered at `b' on a grid of
  size `insize'. Evaluates it at all
  points in the coord matrix, each coordinate having dim
  components. Stores results in outp. If `dflag' is not given or zero,
  evaluates the value. Otherwise: dflag=1,4,16 derivative with respect
  to x,y,z dflag=2,8,32 second derivative with respect to x,y,z

  Usage: outp=mexsplninterpone(b,insize,coord,dim,degree,[dflag]) ;

  $Id: mexsplninterpone.c,v 1.1 1999/11/11 13:15:32 jkybic Exp $

  Jan Kybic, 1999 */

#include <math.h>
#include "mex.h"

/* define a few macros to keep the other source files generic */
#define myErrMsg mexErrMsgTxt
#define NAME "mexsplninterpone"
#define myMalloc(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(double)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myMallocInt(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(int)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myFree(x) mxFree(x)

/* Including source files is not very clean, but works best for generating
   MEX files. Replace by linking for other applications */
#include "bsplneval.c"
#include "splninterp.c"


/* Input Arguments */

#define	B_IN	   prhs[0]
#define INSIZE_IN  prhs[1]
#define	COORD_IN   prhs[2]
#define	DIM_IN	   prhs[3]
#define DEGREE_IN  prhs[4]
#define DFLAG_IN   prhs[5]

/* Output Arguments */

#define	OUTP_OUT	plhs[0]

void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{ double *inp, *coord,*outp ;
  int i,degree,nxi,nyi,nzi,nxc,nyc,nzc,ndc ;
  int bx,by,bz; double *ip ;
  int ndim; const int *dims ;
  int dflag ;

  if (nrhs != 5 && nrhs!=6) { /* Check for proper number of arguments */
    mexErrMsgTxt(NAME " requires five or six input arguments.");
  } else if (nlhs != 1) {
    mexErrMsgTxt(NAME " requires one output argument.");
  }

  for (i=0;i<nrhs;i++) /* Check for the type */
    { if (!mxIsNumeric(prhs[i]) || mxIsComplex(prhs[i]) || 
      mxIsSparse(prhs[i])  || !mxIsDouble(prhs[i])) 
         mexErrMsgTxt(NAME " needs all input arguments to be real.");
     } 
  
  dflag= nrhs==5 ? 0 : (int)(*(mxGetPr(DFLAG_IN))) ;

  degree=(int)(*(mxGetPr(DEGREE_IN))) ;
  ndc=(int)(*(mxGetPr(DIM_IN))) ;
  coord=mxGetPr(COORD_IN) ;
  ip=(double *)(mxGetPr(B_IN)) ;
  bx=(int)ip[0] ; by=ndc>1 ? (int)ip[1] : 0 ; bz=ndc>2 ? (int)ip[2] : 0 ;
  ip=(double *)(mxGetPr(INSIZE_IN)) ;
  nxi=(int)ip[0] ; nyi=(int)ndc>1 ? ip[1] : 0 ; nzi=ndc>2 ? (int)ip[2] : 0 ;

  ndim=mxGetNumberOfDimensions(COORD_IN) ;
  dims=mxGetDimensions(COORD_IN);
  if (ndim>4) mexErrMsgTxt(NAME " unsupported dimensionality.") ;
  if (ndc==1) {
    nxc=dims[0] ; /* the fastest changing index, whichever it is */
    nyc=dims[1] ; nzc=ndim<3 ? 1 : dims[2] ; 
  } else {
    nxc=dims[1] ; /* the fastest changing index, whichever it is */
    nyc=ndim<3 ? 1 : dims[2] ; nzc=ndim<4 ? 1 : dims[3] ; 
  }

  if ((OUTP_OUT=
       mxCreateNumericArray(ndc==1 ? ndim : ndim-1,
       ndc==1 ? dims : dims+1,mxDOUBLE_CLASS,mxREAL))==NULL) 
    mexErrMsgTxt(NAME " Could not create array.") ;
  outp=mxGetPr(OUTP_OUT) ;
 if (splinterpone(bx,by,bz,coord,outp,nxi,nyi,nzi,nxc,nyc,nzc,ndc,degree,dflag))
    myErrMsg(NAME "Error in splinterpone") ;
}
