function [filt,w] = designFilter(filter, len, d, testflag)
% Returns the Fourier Transform of the filter which will be 
% used to filter the projections
%
% INPUT ARGS:   filter - either the string specifying the filter 
%               len    - the length of the projections
%               d      - the fraction of frequencies below the nyquist
%                        which we want to pass
%
% filter= 'ram-lak','shepp-logan' ,'cosine','hamming'  ,'hann'
%         'fract2','fract4','fract6',
%         'bspln0','bspln1','bspln2','bspln3','bspln4',...
%         'bspl0L','bspl1L','bspl2L','bspl3L','bspl4L',...
%
% OUTPUT ARGS:  filt   - the filter to use on the projections

%%% from: /usr/local/matlab/toolbox/images/images/iradon.m

if (nargin<4)
  testflag=0;
end


order = max(64,2^nextpow2(2*len));

% First create a ramp filter - go up to the next highest
% power of 2.

filt = 2*( 0:(order/2) )./order;
w = 2*pi*(0:size(filt,2)-1)/order;   % frequency axis up to Nyquist 

switch filter
case 'ram-lak'
   % Do nothing
case 'sqrt-ramp'
   % |w|^1/2 * |w|^1/2 = |w|
   filt(2:end) = sqrt(filt(2:end));
case 'shepp-logan'
   % be careful not to divide by 0:
   filt(2:end) = filt(2:end) .* (sin(w(2:end)/(2*d))./(w(2:end)/(2*d)));
case 'cosine'
   filt(2:end) = filt(2:end) .* cos(w(2:end)/(2*d));
case 'hamming'  
   filt(2:end) = filt(2:end) .* (.54 + .46 * cos(w(2:end)/d));
case 'hann'
   filt(2:end) = filt(2:end) .*(1+cos(w(2:end)./d)) / 2;
otherwise
   %%%%case FRACTAL FILTERS: 'fract1', 'fract3', 'fract5' ...
   nn=sscanf(filter,'fract%d');
   if(nn>0 & mod(nn,2))
     nn=nn+1;
     filt(2:end) = 2.*sin(w(2:end)/2)./fractsplineautocorr((nn-1)/2,w(2:end)/2/pi)/pi;
     %%% for testing:
     if(testflag==1)
       filt(2:end) = filt(2:end).*(sinc(w(2:end)/pi/2).^(nn));
     end
     
   end
 
   %%% B-spline Filters (truncated):e
   %%% This filter is an oblique projection.
   nn2=sscanf(filter,'bspln%d');
   if(isempty(nn2)==0)
     nn=nn2;
     if(nn>=0)
       nn=nn+1;
       filt(2:end) = filt(2:end)./sinc(filt(2:end)/2).^nn;
       if(testflag==1)
	 filt(2:end) = filt(2:end).*(sinc(w(2:end)/pi/2).^(nn));
       end
     else
       error('Invalid filter selected.');       
     end
   end

   %%% B-spline Filters (LONG FILTER):
   %%% This filter corresponds to a cardinal to bspline transform.
   nn3=sscanf(filter,'bspl%dL');
   if(isempty(nn3)==0)
     nn=nn3;
     if(nn>=0)
       nn=nn+1;
       
       if(nn>=2) % The sum_k ==1 for nn==1 and nn==2!
	 H=filt(2:end).*0;
	 for k=-30:30
	   % sum_k beta(omega+2*pi*k)
	   H=H+sinc(filt(2:end)/2+k).^nn;
	 end
	 filt(2:end) = filt(2:end)./H;
	 if(testflag==1)
	   filt(2:end) = filt(2:end).*(sinc(w(2:end)/pi/2).^(nn));
         end
       end
     else
       error('Invalid filter selected.');       
     end
   end

end

filt(w>pi*d) = 0;                      % Crop the frequency response
filt = [filt' ; filt(end-1:-1:2)'];    % Symmetry of the filter
w=[w,w(end-1:-1:2)]/pi/2';

function show_them()
n=256;d=1;
fft_ramlak=designFilter('ram-lak',n,d);
fft_shepp_logan=designFilter('shepp-logan',n,d);
fft_LP_cosine=designFilter('cosine',n,d);
fft_gen_hamming=designFilter('hamming',n,d);
fft_hann=designFilter('hann',n,d);   

fft_bspln0=designFilter('bspln0',n,d);   
fft_bspln1=designFilter('bspln1',n,d);   


n=n;
f=(0:n/2)/n;

plot(f, fft_ramlak(1:n/2+1),'g-', ...
     f,fft_shepp_logan(1:n/2+1),'b--',...
     f,fft_LP_cosine(1:n/2+1),'r-.',...
     f,fft_gen_hamming(1:n/2+1),'k:' ,...
     f,fft_hann(1:n/2+1),'k-' ,'LineWidth',2)
legend('RAM-LAK','Shepp-Logan','Lowpass cosine','Generalized Hamming','Hann',2);
title('Reconstruction filters');
xlabel('Frequency');
ylabel('Amplitude');
