function  I=radon_accel4_tab_inv(sino, theta, n, model)
% RADON_ACCEL Compute Inverse Radon transform.
%    The RADON function computes the Radon transform, which is the
%    projection of the image intensity along a radial line
%    oriented at a specific angle.
% 
%    R = RADON(I,THETA) returns the Radon transform of the
%    intensity image I for the angle THETA degrees. If THETA is a
%    scalar, the result R is a column vector containing the Radon
%    transform for THETA degrees. If THETA is a vector, then R is
%    a matrix in which each column is the Radon transform for one
%    of the angles in THETA. If you omit THETA, it defaults to
%    0:179.
% 
%    R = RADON(I,THETA,N) returns a Radon transform with the
%    projection computed at N points. R has N rows. If you do not
%    specify N, the number of points the projection is computed at
%    is:
% 
%         2*ceil(norm(size(I)-floor((size(I)-1)/2)-1))+3
%
%    MODEL == 0 - Nearest neighbor
%    MODEL == 1 - Linear interpolation
%    MODEL == 3 - Cubic B-spline interpolation
%    MODEL == N - Interpolation with B-spline of order N
%    MODEL == 1000 - Projection with trikernel N=[0 0 0]
%    MODEL == 1111 - Projection with trikernel N=[1 1 1]
%    MODEL == 1333 - Projection with trikernel N=[3 3 3]
%    MODEL == 1NNM - Projection with trikernel N=[N N M]
%             where N is the degree on the image base function
%             and M on the sinogram base function
%
%    BACK=0  FORWARD RADOn transform;
%    BACK=1  BACK PROJECTION
%
% BEWARE that the center point in both function is the same!
% COMPILE WITH:
%>> mcc -V1.2 -ri radon_accel4_tab_inv.m    
%  

error(nargchk(1,4,nargin))

if(nargin<2)
  theta=[0:179];                 mbreal(theta); % projection angles
end;

theta=(theta./180.*pi);

if(nargin<3 | n==0 | n==[])
   n=floor((size(sino,1)-3)/sqrt(2));
   mbintscalar(n);      % length of sinogram line
end;

if(nargin<4)
  model=1;                        % used model;
end;


   T=size(sino,2);                 mbintscalar(T);    % number of angles
   if(T~=length(theta))
     error('number of angle in sinogram is different');
   end;
   
   N=size(sino,1);

   sinoline=zeros(N,1);            mbreal(sinoline);
   
   I=zeros(n,n);  mbreal(I);

   tim=0;

   %% LOOP ON ALL ANGLES THETA %%%%%%%
   for i=[1:T];                    mbintscalar(i);
     tic;
     angle=theta(i);                 mbrealscalar(angle);
     disp([i,angle/pi*180,tim])
     ct=abs(cos(angle));                 mbrealscalar(ct);
     st=abs(sin(angle));                 mbrealscalar(st);
     if(ct<1E-5) ct=0; end;
     if(st<1E-5) st=0; end;

     %% Support for kernel:
     nn=0;h1=1;                mbintscalar(nn);mbrealscalar(h1);
     support=(((nn+1)*h1+(nn+1)*ct+(nn+1)*st)); mbrealscalar(st);

     
     sinoline=zeros(N,1);            mbreal(sinoline);
     
     d11=distance(1,1,angle,n,N);      mbrealscalar(d11);
     d1n=distance(1,n,angle,n,N);      mbrealscalar(d1n);
     dn1=distance(n,1,angle,n,N);      mbrealscalar(dn1);
     dnn=distance(n,n,angle,n,N);      mbrealscalar(dnn);
     
     dxy1=d11;
     dxy1step=(dn1-d11)./(n-1);
     dxyn=d1n;
     dxynstep=(dnn-d1n)./(n-1);

     %%% Using the code from nkernel_tab
     % calculate the Splat at fine resolution
     %
     if (model>=1000 & model<1444)
       res=0.002;
       if(model==1111)
	 NN=[1 1 1];
       elseif(model==1000)
	 NN=[0 0 0];
       elseif(model==1333)
	 NN=[3 3 3];
	 res=0.01;
       elseif(model>1000)
	 NN=floor([mod(model,1000)/100 mod(model,100)/10 mod(model,10)]);
       end
       HH=[ct st 1];
       support2=sum((NN+1).*HH)./2;
       xxx=[-support2:res:support2];  mbreal(xxx);
       yyy=nkernel(NN,HH,xxx);        mbreal(yyy);
       yyy=[0 yyy 0];
     end     
     
     sinoline=sino(:,i);

     %% LOOP OVER ALL X
     for x=1:n
       dy=dxy1;                      mbrealscalar(dy);
       dystep=(dxyn-dxy1)./(n-1);    mbrealscalar(dystep);

       %% LOOP OVER ALL Y
       for y=1:n
	 d=dy;                       mbrealscalar(d);
	 
%	 point=I(x,y);             mbrealscalar(point);

	 %% IF VALID POINTS
	 if(d>=1 & d<=N)

	   %%% CHOOSE INTERPOLATION MODEL
	   if(model==0)       % nearest neighbor
	     xx=round(d);              mbreal(xx);
	     %sinoline(xx)=sinoline(xx)+I(x,y);
	     I(x,y)=I(x,y)+sinoline(xx);

	   elseif(model==1)   % linear interpolation

	     ceild=ceil(d);            mbintscalar(ceild);
	     floord=ceild-1;           mbintscalar(floord);
	     linear=ceild-d;           mbrealscalar(linear);
	     
	     %sinoline(ceild)=sinoline(ceild)+point*(1-linear);
	     %sinoline(floord)=sinoline(floord)+point*(linear);	     
	     I(x,y)=I(x,y)+sinoline(ceild)*(1-linear)+sinoline(floord)*(linear);
	   
	   elseif(model>=2 & model<30)   % n-th B-spline interpolation

	     [xx,yy]=bsplns(d,model);
	     xx=round(d-xx);
	     
	     yy(xx<=0|xx>N)=[];
	     xx(xx<=0|xx>N)=[];
	     %sinoline(xx)=sinoline(xx)+I(x,y)*yy';	
	     I(x,y)=I(x,y)+sum(sinoline(xx).*yy');	     
	   
	   elseif(model==-1000103)   % cubic moms

	     [xx,yy]=comos(d);
	     xx=round(d-xx);
	     
	     yy(xx<=0|xx>N)=[];
	     xx(xx<=0|xx>N)=[];
	     %sinoline(xx)=sinoline(xx)+I(x,y)*yy';	
	     I(x,y)=I(x,y)+sum(sinoline(xx).*yy');	     

	   elseif(model==-1000133)   % cubic keys

	     [xx,yy]=ckeys(d);
	     xx=round(d-xx);
	     
	     yy(xx<=0|xx>N)=[];
	     xx(xx<=0|xx>N)=[];
	     %sinoline(xx)=sinoline(xx)+I(x,y)*yy';	
	     I(x,y)=I(x,y)+sum(sinoline(xx).*yy');	     

	   elseif(model>=1000 & model <1444) 	     
	     % B-spline least-square projection onto B-spline line

	     xx=mod(d,1)+[-ceil(support/2):ceil(support/2)];

 	     %Instead of:
	     %  yy=nkernel_tab([1,1,1],[ct,st,1],xx);

	     %% BEGIN: USE CODE DIRECT from nkernel_tab.m
	     xxr=((xx+(support2))./res+2); mbreal(xxr);
	     xxr(xxr<2)=1; 
	     xxr(xxr>length(yyy))=1;
	     xxfloor=floor(xxr);           mbint(xxfloor);
	     xxr=xxr-xxfloor;
	     yy=yyy(xxfloor).*(1-xxr)+yyy(xxfloor+1).*xxr;
	     %% END USE CODE DIRECT
	     
	     xx=round(d-xx);               mbreal(xx);
	     
	     yy(xx<=0|xx>N)=[];
	     xx(xx<=0|xx>N)=[];

	     %sinoline(xx)=sinoline(xx)+I(x,y)*yy';	     
	     I(x,y)=I(x,y)+sum(sinoline(xx).*yy');	     

           end;
	   %%% END CHOOSE INTERPOLATION MODEL
	   
	 %%ENDIF VALID POINTS
	 end;

	 dy=dy+dystep;	 
	 %%ENDLOOP OVER ALL Y
       end; 

       dxy1=dxy1+dxy1step;
       dxyn=dxyn+dxynstep;
       %% ENDLOOP OVER ALL X
     end;

     tim=toc;
     %% ENDLOOP OVER ALL ANGLES
   end;    

I=I.*n./T*pi/2;
return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function d=distance(x,y,angle,n,N)  
   mbintscalar(x); mbintscalar(y); mbrealscalar(angle); mbintscalar(n);mbintscalar(N);

%   x0=x-n/2-0.0000001-1;         mbrealscalar(x0);
%   y0=y-n/2-0.0000001-.5;        mbrealscalar(y0);

x0=x-n/2-0.5 ;         mbrealscalar(x0);
y0=y-n/2-0.5 ;         mbrealscalar(y0);

   beta=atan2(y0,x0);            mbscalar(beta);
   alpha=angle-beta;             mbscalar(alpha);
   
   if (alpha>pi)
     alpha=alpha-pi-pi;
   end;
   
   r=sqrt(x0.*x0+y0.*y0);        mbrealscalar(r);
   
   rp=cos(alpha).*r;             mbrealscalar(rp);
   
   xp=rp.*cos(angle);             mbrealscalar(xp);
   
   yp=rp.*sin(angle);             mbrealscalar(yp);
   
   d=sqrt((x0-xp).*(x0-xp)+(y0-yp).*(y0-yp));       mbrealscalar(d);
   
   d=N/2-d.*sign(alpha);        mbrealscalar(d);
return;   


function y=cbspln(x)
% BSPLN3   B-spline of degree 3 (cubic) 
%    BSPLN3(X) is the value of a B-spline of degree 3 at point x
%    Method: Polynom evaluation
%
%    SEE also: BSPLN, BSPLN0, BSPLN1,...,BSPLN7
mbrealscalar(x);
x=abs(x);
if x>2,
  y=0 ;
else
  if x>1,
    y=(2-x)^3/6 ;
  else
    y=2/3-x^2*(1-x/2) ;
  end ;
end ;
mbrealscalar(y);



function [x,y]=bsplns(x,n)
% BSPLNS(X,N)   Centered B-spline of degree N
%    BSPLNS(X,N) calculate centered B-spline of degree n at point x+k
%    Method: Recursive formular (by T.Blu)
%
%    [Y,X]=BSPLNS(X,N) retunr values Y and position X of points x+k
%
%    See also: NBSPLN, BSPLN0, BSPLN1,...,BSPLN7, BSPLN, BSPLNS

mbrealscalar(x);
mbintscalar(n);

% Order L is degree N+1:
L=n+1;
% Convert to centered B-spline 
x=x+L/2;
x0=sort(x-floor(x));
y=ones(1,length(x0));
z=zeros(1,length(x0));
x=x0;
for k=2:L
  y=[y z];
  x=[x0 x+1];
  y=(x.*y+(k-x).*[z y(1:(length(y)-length(z)))])/(k-1);
end;

x=x-L/2;

if nargout<=1
        x=y;
end


